"""

This module provides the complete basic ChemApp library API for use in Python
programs.
"""
from __future__ import annotations
import builtins as __builtins__
from typing import List, Literal, Tuple
from chemapp.core import AmountUnit
from chemapp.core import ChemAppError
from chemapp.core import ConditionVariable
from chemapp.core import ConfigurationOption
from chemapp.core import EnergyUnit
from chemapp.core import FunctionData
from chemapp.core import FunctionSumVariable
from chemapp.core import InteractionVariable
from chemapp.core import IoOption
from chemapp.core import LimitVariable
from chemapp.core import PhaseConstituentData
from chemapp.core import PhaseConstituentVariable
from chemapp.core import PhaseMapVariable
from chemapp.core import PressureUnit
from chemapp.core import Quantity
from chemapp.core import ResultVariable
from chemapp.core import SolutionModel
from chemapp.core import Status
from chemapp.core import StreamVariable
from chemapp.core import TargetVariable
from chemapp.core import TemperatureUnit
from chemapp.core import VolumeUnit
import logging as logging
import os as os
from pathlib import Path

__all__ = [
    "AmountUnit",
    "ChemAppError",
    "ConditionVariable",
    "ConfigurationOption",
    "EnergyUnit",
    "FunctionData",
    "FunctionSumVariable",
    "InteractionVariable",
    "IoOption",
    "LimitVariable",
    "PhaseConstituentData",
    "PhaseConstituentVariable",
    "PhaseMapVariable",
    "PressureUnit",
    "Quantity",
    "ResultVariable",
    "SolutionModel",
    "Status",
    "StreamVariable",
    "TargetVariable",
    "TemperatureUnit",
    "VolumeUnit",
    "disable_logging",
    "enable_logging",
    "logging",
    "tqbond",
    "tqcdat",
    "tqce",
    "tqcel",
    "tqcen",
    "tqcenl",
    "tqcfct",
    "tqchar",
    "tqcio",
    "tqclim",
    "tqclos",
    "tqcnsc",
    "tqconf",
    "tqcprt",
    "tqcsp",
    "tqcspc",
    "tqcsu",
    "tqcsum",
    "tqefct",
    "tqerr",
    "tqgdat",
    "tqgdpc",
    "tqgetr",
    "tqgfct",
    "tqgio",
    "tqgnlc",
    "tqgnp",
    "tqgnpc",
    "tqgnsc",
    "tqgpar",
    "tqgsp",
    "tqgspc",
    "tqgsu",
    "tqgsum",
    "tqgted",
    "tqgthi",
    "tqgtid",
    "tqgtlc",
    "tqgtnm",
    "tqgtpi",
    "tqgtrh",
    "tqifct",
    "tqini",
    "tqinlc",
    "tqinp",
    "tqinpc",
    "tqinsc",
    "tqlite",
    "tqlpar",
    "tqmap",
    "tqmapl",
    "tqmodl",
    "tqnexc",
    "tqnfct",
    "tqnolc",
    "tqnop",
    "tqnopc",
    "tqnosc",
    "tqnosl",
    "tqnpar",
    "tqopen",
    "tqopna",
    "tqopnt",
    "tqpcis",
    "tqrcst",
    "tqremc",
    "tqrfil",
    "tqsetc",
    "tqshow",
    "tqsize",
    "tqstca",
    "tqstec",
    "tqstpc",
    "tqstrm",
    "tqstsc",
    "tqsttp",
    "tqstxp",
    "tqused",
    "tqvers",
    "tqwasc",
    "tqwstr",
]

def disable_logging() -> None:
    """
    Turn off logging of ChemApp calls to 'calls.log'
    """

def enable_logging() -> None:
    """
    Set the ChemApp calls to be output including returns to 'calls.log'
    """

def tqbond(indexp: int, indexa: int, indexb: int, indexc: int, indexd: int) -> float:
    """
    Calculate a quadruplet or pair fraction (only for the models SUBG, QUAS and
    QSOL).

    Note that the second sublattice constituent index = (total number of
    constituents of the first sublattice + the second sublattice constituent
    index). The index numbers may be entered in any order, but two sublattice
    constituents must belong to the first sublattice and two to the second

    Parameters
    ----------
    indexp
        zero-based phase index
    indexa
        zero-based sublattice constituent index
    indexb
        zero-based sublattice constituent index
    indexc
        zero-based sublattice constituent index
    indexd
        zero-based sublattice constituent index

    Returns
    -------
    float
        quadruplet or pair fraction

    """

def tqcdat(i1: int, i2: int, i3: int, i4: int, i5: int, val: float) -> int:
    """
    Change selected data of a specified phase constituent or of a specified
    phase from a thermodynamic data-file in ASCII format. If a phase is
    included with more than one copy in the data-file, then all copies of the
    phase will be identically changed.

    Parameters
    ----------
    i1
        zero-based index number as described below
    i2
        zero-based index number as described below
    i3
        zero-based index number as described below
    i4
        zero-based index number as described below
    i5
        zero-based index number as described below
    val
        new value of the selected quantity for the specified phase constituent
        of for the specified phase


    Values of i1-i5 required when calling TQCDAT:

    * Enthalpy: ``1, 0, 0, I, M``

      ``I`` = number of phase constituent

      ``M`` = number of phase

    * Entropy: ``1, 0, 1, I, M``

      ``I`` = number of phase constituent

      ``M`` = number of phase

    * Heat capacity: ``1, INDEXR, N, I, M``

      ``INDEXR`` = number of Cp expression

      ``N`` = number of term in the Cp expression

      ``I`` = number of phase constituent

      ``M`` = number of phase

      __Note__: The Cp expression will normally contain 4 terms,
      Cp = a + b*T + c*T2 + d/T2, but can also include six terms of variable
      powers of temperature. The maximum value of N is thus 10. For the models
      PIHZ, HELZ, HTSZ, HTWZ and HTDZ, one expression (INDEXR=1) contains
      seven Helgeson terms. Heat capacity data will be returned also for
      files with stored values of standard Gibbs energy.

    * Heat of transformation: ``1, INDEXR, N, I, M``

      ``INDEXR`` = number of Cp expression (this number must always be less than
      the total number of Cp expressions)

      ``N`` = the total number of terms in the Cp expressions + 1

      ``I`` = number of phase constituent

      ``M`` = number of phase

    * Molar volume: ``4, 1, 1, I, M``

      ``I`` = number of phase constituent

      ``M`` = number of phase

    * Coefficients of the equation of state: ``4, INDEXR, N, I, M``

      ``INDEXR = 2``: thermal expansion (= a + b*T + c/T + d/T2)

      ``INDEXR = 3``: compressibility (= a + b*T + c*T2 + d*T3) or
      bulk modulus (= a + b*(T-T0) + c*(T2-T02) + d*(1/T-1/T0)), depending on
      the equation of state being used

      ``INDEXR = 4``: pressure derivative of the bulk modulus
      (= a + b*(T-T0)*ln(T/T0))

      ``N`` = number of coefficient

      ``I`` = number of phase constituent

      ``M`` = number of phase

    * Critical temperature of a constituent of a gas phase modelled with IDVT or

      IDVD: ``4, 1, 1, I, 1``

      ``I`` = number of phase constituent

    * Critical pressure of a constituent of a gas phase modelled with IDVT or

      IDVD: ``4, 1, 2, I, 1``

      ``I`` = number of phase constituent

    * Critical volume of a constituent of a gas phase modelled with IDVT or

      IDVD: ``4, 1, 3, I, 1``

      ``I`` = number of phase constituent

    * Acentric factor of a constituent of a gas phase modelled with IDVT or

      IDVD: ``4, 1, 4, I, 1``

      ``I`` = number of phase constituent

    * Dipole moment of a constituent of a gas phase modelled with IDVD:

      ``4, 1, 5, I, 1``

      ``I`` = number of phase constituent

    * Curie/Neel temperature of a phase constituent of a magnetic phase:

      ``7, 1, 1, I, M``

      ``I`` = number of phase constituent

      ``M`` = number of phase

    * Average magnetic moment/atom of a phase constituent of a magnetic phase:

      ``7, 1, 2, I, M``

      ``I`` = number of phase constituent

      ``M`` = number of phase

    * Excess Curie temperature term of a magnetic phase:

      ``10, INDEXX, NOEXPR, 1, M``

      ``INDEXX`` = number of interaction according to the order of the datafile

      ``NOEXPR`` = number of Redlich-Kister expression of interaction INDEXX

      ``M`` = number of phase

      **Note**: ``NOEXPR = 1`` for the Redlich-Kister term with power zero, and so
      on.

    * Excess magnetic moment term of a magnetic phase:

      ``10, INDEXX, NOEXPR, 2, M``

      ``INDEXX`` = number of interaction according to the order of the datafile

      ``NOEXPR`` = number of Redlich-Kister expression of interaction INDEXX

      ``M`` = number of phase

      **Note**: ``NOEXPR = 1`` for the Redlich-Kister term with power zero, and so
      on.

    * Excess Gibbs energy coefficient: ``13, INDEXX, NOEXPR, N, M``

      ``INDEXX`` = number of interaction according to the order of the datafile

      ``NOEXPR`` = number of expression of interaction INDEXX

      ``N`` = number of term

      ``M`` = number of phase

      **Note**: The standard excess Gibbs energy expression contains maximum 6
      terms of a + b*T + c*T*ln(T) + d*T2 + e*T3 + f/T (+g*P + h*P2).
      ``NOEXPR = 1`` for the Redlich-Kister term with power zero, and so on. For
      phases not containing any Redlich-Kister terms and SUBG excepted,
      ``NOEXPR`` is one. SUBG involves excess Gibbs energy expressions containing
      six terms with variable powers of temperature. For any of these terms
      ``NOEXPR`` = 2. Excess Gibbs energy expressions with factor P (= pressure)
      have term numbers (MIT+1) to (MIT+MIT) where MIT is the number of
      temperature dependent terms used in the standard expression.

    * Stoichiometry: ``16 ,1, INDEXC, I, M``

      ``INDEXC`` = number of system component

      ``I`` = number of phase constituent

      ``M`` = number of phase

      **Note**: Since changing the stoichiometry might lead to erroneous
      calculations for several solution models, this option is intended for
      expert users only.
    """

def tqce(
    option: TargetVariable,
    indexp: int = -1,
    indexc: int = -1,
    vals: Tuple[float, float] | None = None,
) -> None:
    """
    Calculate the equilibrium with the current settings of global conditions or
    streams.

    Parameters
    ----------
    option
        option indicating the target variable for the calculation
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    vals
        initial estimate of the target variable, or the lower and upper limits
        of an incoming amount, if necessary



    """

def tqcel(
    option: TargetVariable,
    indexp: int = -1,
    indexc: int = -1,
    vals: Tuple[float, float] | None = None,
) -> None:
    """
    Calculate the equilibrium with current settings of global conditions or
    streams, and list the results in the form of a ChemSage format output table.

    Parameters
    ----------
    option
        option indicating the target variable for the calculation
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    vals
        initial estimate of the target variable, or the lower and upper limits
        of an incoming amount, if necessary



    """

def tqcen(
    option: TargetVariable,
    indexp: int = -1,
    indexc: int = -1,
    vals: Tuple[float, float] | None = None,
) -> None:
    """
    Calculate the equilibrium, taking results from the previous equilibrium
    calculation as initial estimates

    Parameters
    ----------
    option
        option indicating the target variable for the calculation
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    vals
        initial estimate of the target variable, or the lower and upper limits
        of an incoming amount, if necessary



    """

def tqcenl(
    option: TargetVariable,
    indexp: int = -1,
    indexc: int = -1,
    vals: Tuple[float, float] | None = None,
) -> None:
    """
    Calculate the equilibrium, taking results from the previous equilibrium
    calculation as initial estimates, and list the results in the form of a
    ChemSage format output table.

    Parameters
    ----------
    option
        option indicating the target variable for the calculation
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    vals
        initial estimate of the target variable, or the lower and upper limits
        of an incoming amount, if necessary



    """

def tqcfct(
    indexf: int,
    option: FunctionData,
    indexc: int,
    indexr: int,
    indexx: int,
    value: float,
) -> None:
    """
    Change a coefficient in a function.


    .. note::
        You can only use this function if the ChemSage.dat contains functions,
        which is available when exporting from FactSage 8.1+

    Parameters
    ----------
    indexf
        Index number for the function as obtained by calling ``TQIFCT``/``TQNFCT``
        **For Volume only:** Index number for the solution phase
    option
        property type (FunctionData enumeration)
        **For Volume only** Index number for the constituent of phase indexf
    indexc
        **For Volume only** zero-based function index, only used for V
    indexr
        Index number for the Cp range, not being used for H and S
        **For Volume only** Index number for the molar volume function as
        obtained by calling ``TQIFCT``/``TQNFCT``
    indexx
        Index number for a coefficient

        This can be maximum 4 when no extra powers are entered,
        else up to ``[(number of extra powers)+4]``. The number of
        extra powers is maximum 6. The extra powers cannot be modified.
        **Only affecting Cp and Volume**
    value
        New value of the coefficient

    """

def tqchar(indexp: int, indexc: int) -> float:
    """
    Get the charge of a phase constituent.

    Parameters
    ----------
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index


    Returns
    -------
    float
        phase constituent charge
    """

def tqcio(option: IoOption, ival: int) -> None:
    """
    Redirect input and output from ChemApp to another unit than the default, or
    select a different language for the error messages than the default.


    Parameters
    ----------
    option
        input/output option
    ival
        a legal unit number, or to a language index

    """

def tqclim(option: LimitVariable, val: float) -> None:
    """
    Modify the upper and lower limits of the target variables pressure,
    volume or temperature.

    Find more information on the available values for `LimitVariable` in the
    :ref:`chemapp.core.LimitVariable` documentation.

    Parameters
    ----------
    option
        target variable limit to set
    val
        new value for the limit
    """

def tqclos(unit: int) -> None:
    """
    Close a file that has been previously opened using TQOPNA, TQOPNB, TQOPNT,
    or TQOPEN.

    Parameters
    ----------
    unit
        unit number under which the file was opened
    """

def tqcnsc(indexs: int, name: str) -> None:
    """
    Change the name of the specified system component.

    Parameters
    ----------
    indexs
        zero-based system component index
    name
        new name
    """

def tqconf(
    option: ConfigurationOption, valuea: int = -1, valueb: int = -1, valuec: int = -1
) -> None:
    """
    Set a configuration option.

    Parameters
    ----------
    option
        configuration option
    valuea
        configuration value A
    valuea
        configuration value B
    valuea
        configuration value C
    """

def tqcprt() -> str:
    """
    Get the ChemApp copyright message.

    Returns
    -------
    str
        ChemApp copyright message
    """

def tqcsp(indexp: int, status: Status) -> None:
    """
    Change the status of the specified phase.

    Find more information on the available values for `Status` in the
    :ref:`chemapp.core.Status` documentation.


    Parameters
    ----------
    indexp
        zero-based phase index
    status
        new phase status
    """

def tqcspc(indexp: int, indexc: int, status: Status) -> None:
    """
    Change the status of a specified phase constituent.

    Find more information on the available values for `Status` in the
    :ref:`chemapp.core.Status` documentation.


    Parameters
    ----------
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    status
        new phase constituent status

    """

def tqcsu(
    option: Quantity,
    unit: PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit,
) -> None:
    """
    Change the unit for the specified quantity.

    Find more information on the available values for the units  in the
    respective parts of the documentation:

    * Option:  :ref:`chemapp.core.Quantity`
    * PressureUnit: :ref:`chemapp.core.PressureUnit`
    * VolumeUnit: :ref:`chemapp.core.VolumeUnit`
    * TemperatureUnit: :ref:`chemapp.core.TemperatureUnit`
    * EnergyUnit: :ref:`chemapp.core.EnergyUnit`
    * AmountUnit: :ref:`chemapp.core.AmountUnit`


    Parameters
    ----------
    option
        the quantity of which the unit is changed
    unit
        the new unit


    """

def tqcsum(
    option: FunctionSumVariable, indexp: int, indexc: int, nval: int, value: float
) -> None:
    """
    Change a pre-factor of a function for an phase constituent.

    .. note::
        You can only use this function if the ChemSage.dat contains functions,
        which is available when exporting from FactSage 8.1+

    The pre-factor is the coefficient of the function in the Gibbs energy
    function. The existing factors can be listed with :ref:`chemapp.basic.tqgsum`.

    Parameters
    ----------
    option
         Either Gibbs ('G') or volume ('V') functions
    indexp
        zero-based phase index
    indexc
        zero-based constituent index
    nval
        zero-based index of the factor in the list of functions
    value
        new value for the pre-factor
    """

def tqefct(indexf: int) -> int:
    """
    Get the number of expressions of a function.

    .. note::
        You can only use this function if the ChemSage.dat contains functions,
        which is available when exporting from FactSage 8.2+

    Parameters
    ----------
    indexf
        zero-based function index

    Returns
    -------
    str
        number of expressions
    """

def tqerr() -> str:
    """
    Get the current error message. All ChemApp error messages are written
    sequentially to the unit/file associated with ERROR. Use tqcio to change
    the associated unit/file and on how to prevent the automatic output of
    error messages.

    .. note::
        This routine typically does not need to be called actively. Any error
        that is interrupting program flow will be raised as an expression,
        containing the error message that tqerr() would yield.

    Returns
    -------
    str
        error message
    """

def tqgdat(
    indexp: int, indexc: int, option: PhaseConstituentData, indexr: int = 0
) -> List[float]:
    """
    Get selected thermodynamic data of a specified phase constituent loaded
    from a thermodynamic data-file in ASCII format.

    Parameters
    ----------
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    option
        option
    indexr
        zero-based index number of a Cp expression (only used for options Cp,
        Tt, and Ht)

    Returns
    -------
    List[float]
        thermodynamic coefficients
    """

def tqgdpc(option: PhaseConstituentVariable, indexp: int, indexc: int) -> float:
    """
    Get thermodynamic data for a phase constituent.

    Find more information on the available values for `PhaseConstituentVariable` in the
    :ref:`chemapp.core.PhaseConstituentVariable` documentation.


    Parameters
    ----------
    option
        option identifying the thermodynamic quantity
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index

    Returns
    -------
    float
        thermodynamic quantity. The return value (for a single constituent,
        selected using INDEXP and INDEXC) is dimensionless and is calculated for
        the default temperature, or for the last temperature set with TQSETC or
        TQSTEC, or calculated using TQCE/TQMAP

    Note
    ----
    To obtain quantities having dimensions, the return value needs to be
    multiplied by R*T (gas constant times temperature).
    """

def tqgetr(
    option: ResultVariable, indexp: int = -1, index: int = -1
) -> float | List[float]:
    """
    Get calculation results obtained by the last call to tqce, tqcel, tqmap, or
    tqmapl.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.ResultVariable


    Parameters
    ----------
    option
        option indicating the state variable to retrieve
    indexp
        zero-based phase index
    index
        zero-based phase constituent or system component index

    Returns
    -------
    float|List[float]
        retrieved float value or array of float values. ``indexp`` and ``index``
        values determine when a single value or array is returned. See the
        table below for an explanation.

        More information:

        https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/basic/index.html#chemapp.basic.tqgetr


    """

def tqgfct(indexf: int, option: FunctionData, indexc: int = 0, indexr: int = 0) -> None:
    """
    Get a property of the specified function.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.FunctionData


    .. note::
        You can only use this function if the ChemSage.dat contains functions,
        which is available when exporting from FactSage 8.1+

    Parameters
    ----------
    indexf
        Index number for the function as obtained by calling ``TQIFCT``/``TQNFCT``
        **For Volume only:** Index number for the solution phase
    option
        property type
        **For Volume only** Index number for the constituent of phase indexf
    indexc
        **For Volume only** zero-based function index, only used for V, default=0
    indexr
        Index number for the Cp range, not being used for H and S, default=0
        **For Volume only** Index number for the molar volume function as
        obtained by calling ``TQIFCT``/``TQNFCT``
    """

def tqgio(option: IoOption) -> int:
    """
    Get the unit number and language used for error messages, and the unit
    number for the thermodynamic data-file and optional output lists,
    respectively.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.IoOption


    Parameters
    ----------
    option
        input/output option

    Returns
    -------
    int
        file unit number or language index

    """

def tqgnlc(indexp: int, indexl: int, indexc: int) -> str:
    """
    Get the name of the specified sublattice constituent.

    Parameters
    ----------
    indexp
        zero-based phase index
    indexl
        zero-based sublattice index
    indexc
        zero-based sublattice constituent index

    Returns
    -------
    str
        sublattice constituent name
    """

def tqgnp(indexp: int) -> str:
    """
    Get the name of a specified phase.

    Parameters
    ----------
    indexp
        zero-based phase index

    Returns
    -------
    str
        phase name
    """

def tqgnpc(indexp: int, indexc: int) -> str:
    """
    Get the name of the specified phase constituent.

    Parameters
    ----------
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index

    Returns
    -------
    str
        phase constituent name
    """

def tqgnsc(indexs: int) -> str:
    """
    Get the name of the specified system component.

    Parameters
    ----------
    indexs
        zero-based system component index

    Returns
    -------
    str
        phase constituent name
    """

def tqgpar(
    indexp: int, object: InteractionVariable, indexx: int
) -> Tuple[int, int, List[float]]:
    """
    Get selected excess Gibbs energy or excess magnetic parameters of a
    specified phase from a thermodynamic data-file in ASCII format.

    Parameters
    ----------
    indexp
        zero-based phase index
    option
        interaction variation, either ('G') excess Gibbs energy
        interactions or ('M') excess magnetic interactions
    indexx
        zero-based interaction index

    Returns
    -------
    int
        ``noexpr``, the number of temperature-dependent expressions of interaction indexx.
    int
        ``nvala``, the number of values of each temperature dependent expression.
    List[float]
        ``'noexpr'`` by ``'nvala'`` values, i.e. ``'noexpr'*'nvala'`` values of the quantity requested.
    """

def tqgsp(indexp: int) -> Status:
    """
    Get the status of a specified phase.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.Status


    Parameters
    ----------
    indexp
        zero-based phase index


    Returns
    -------
    Status
        status

    """

def tqgspc(indexp: int, indexc: int) -> Status:
    """
    Get the status of the specified phase constituent.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.Status


    Parameters
    ----------
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index

    Returns
    -------
    Status
        phase constituent status

    """

def tqgsu(
    option: Quantity,
) -> PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit:
    """
    Get the unit that is currently in use for a specified quantity.

    +-------------+-------+
    | Quantity    | Unit  |
    +=============+=======+
    | Pressure    | bar   |
    |             +-------+
    |             | atm   |
    |             +-------+
    |             | Pa    |
    |             +-------+
    |             | kPa   |
    |             +-------+
    |             | psi   |
    |             +-------+
    |             | torr  |
    +-------------+-------+
    | Volume      | dm3   |
    |             +-------+
    |             | cm3   |
    |             +-------+
    |             | m3    |
    |             +-------+
    |             | ft3   |
    |             +-------+
    |             | in3   |
    +-------------+-------+
    | Temperature | K     |
    |             +-------+
    |             | C     |
    |             +-------+
    |             | F     |
    +-------------+-------+
    | Energy      | J     |
    |             +-------+
    |             | cal   |
    |             +-------+
    |             | Btu   |
    |             +-------+
    |             | kWh   |
    +-------------+-------+
    | Amount      | mol   |
    |             +-------+
    |             | gram  |
    |             +-------+
    |             | kg    |
    |             +-------+
    |             | tonne |
    |             +-------+
    |             | poudn |
    +-------------+-------+

    Parameters
    ----------
    option
        the quantity for which the current unit is requested

    Returns
    -------
    unit
        unit being used


    """

def tqgsum(
    option: FunctionSumVariable, indexp: int, indexc: int
) -> Tuple[int, List[float], List[str]]:
    """
    List all functions and pre-factors of a phase constituent that make the
    Gibbs energy function or volume function

    The lists ``factors`` and ``names`` are of the exact length of ``nofct``.

    Parameters
    ----------
    option
        ('G') Gibbs energy functions or ('V') volume functions
    indexp
        zero-based phase index
    indexc
        zero-based constituent index

    Returns
    -------
    int
        total number of functions used
    List[float]
        pre-factors / coefficients of the used functions
    List[str]
       names of functions

    .. note::
        You can only use this function if the ChemSage.dat contains functions,
        which is available when exporting from FactSage 8.1+

    """

def tqgted() -> int:
    """
    Get the ChemApp license expiration date.

    Returns
    -------
    int
        license expiry date
    """

def tqgthi() -> Tuple[str, int]:
    """
    Get the HASP dongle type and id.

    Returns
    -------
    Tuple[str,int]
        (dongle type, dongle id)
    """

def tqgtid() -> str:
    """
    Get the user ID of the ChemApp library license holder.

    Returns
    -------
    str
        User ID
    """

def tqgtlc(indexp: int, indexl: int, indexc: int) -> float:
    """
    Get the calculated equilibrium sublattice site fraction.

    Parameters
    ----------
    indexp
        zero-based phase index
    indexl
        zero-based sublattice index
    indexc
        zero-based sublattice constituent index

    Returns
    -------
    float
        sublattice site fraction
    """

def tqgtnm() -> str:
    """
    Get the name of the ChemApp library license holder.

    Returns
    -------
    str
        ChemApp user name
    """

def tqgtpi() -> str:
    """
    Get the program ID.

    Returns
    -------
    str
        Program ID
    """

def tqgtrh() -> (
    Tuple[int, str, List[int], str, List[int], List[int], List[int], str, str, str]
):
    """
    Retrieve information stored in the header of a transparent file.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/basic/index.html#chemapp.basic.tqgtrh


    Returns
    -------
    Tuple[int, str, List[int], str, List[int], List[int], List[int], str, str, str]
        (ver, nwp, vnw, nrp, vnr, dtc, dte, id, usr, rem) as described in the table.
    """

def tqifct(indexf: int) -> str:
    """
    Get the name of a function.

    Parameters
    ----------
    indexf
        index of the function

    Returns
    -------
    str
        name of the function

    Note
    ----
    You can only use this function if the ChemSage.dat contains functions,
    which is available when exporting from FactSage 8.1+

    """

def tqini() -> None:
    """
    Use this method to initialise ChemApp.

    It must be called before any other ChemApp subroutine, or whenever you wish
    to reinstate the complete set of default values and units, and read a
    thermodynamic data-file.
    """

def tqinlc(name: str, indexp: int, indexl: int) -> int:
    """
    Get the index number of the specified sublattice constituent.

    Parameters
    ----------
    name
        sublattice constituent name
    indexp
        zero-based phase index
    indexl
        zero-based sublattice index

    Returns
    -------
    int
        zero-based index
    """

def tqinp(name: str) -> int:
    """
    Get the zero-based index number of a specified phase.

    Parameters
    ----------
    name
        phase name


    Returns
    -------
    int
        zero-based index
    """

def tqinpc(name: str, indexp: int) -> int:
    """
    Get the zero-based index number of the specified phase constituent.

    The required ``indexp`` can be found using ``tqinp()``.

    Parameters
    ----------
    name
        phase constituent name
    indexp
        zero-based phase index

    Returns
    -------
    int
        zero-based index
    """

def tqinsc(name: str) -> int:
    """
    Get the zero-based index number of the specified system component.

    Parameters
    ----------
    name
        system component name

    Returns
    -------
    int
        zero-based index
    """

def tqlite() -> bool:
    """
    Check whether the type of ChemApp currently used is ChemApp light.

    Returns
    -------
    bool
        Light Version
    """

def tqlpar(
    indexp: int, object: InteractionVariable
) -> Tuple[int, List[str], List[int]]:
    """
    List all excess Gibbs energy or all excess magnetic interactions of a
    specified phase from a thermodynamic data-file in ASCII format.

    The third item of the returned tuple will likely be removed in the future.
    It is obsolete.

    Parameters
    ----------
    indexp
        zero-based phase index (int)
    option
        interaction variation ('G') excess Gibbs energy interactions or ('M')
        excess magnetic interactions

    Returns
    -------
    int
        the total number of interactions.
    List[str]
        all interaction strings
    List[int]
        lengths of each of the previous strings
    """

def tqmap(
    option: PhaseMapVariable, indexp: int, indexc: int, vals: List[float]
) -> bool:
    """
    Perform a one-dimensional phase mapping calculation, giving all phase
    transitions within a defined interval.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.PhaseMapVariable


    Parameters
    ----------
    option
        option indicating the search variable for the calculation
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    vals
        array of two floats, the lower and upper limits of the search variable

    Returns
    -------
    bool
        whether more calls to tqmap is necessary

    """

def tqmapl(
    option: PhaseMapVariable, indexp: int, indexc: int, vals: List[float]
) -> bool:
    """
    Perform a one-dimensional phase mapping calculation and list the results
    in the form of a ChemSage format output table.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.PhaseMapVariable


    Parameters
    ----------
    option
        option indicating the search variable for the calculation
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    vals
        array of two floats, the lower and upper limits of the search variable

    Returns
    -------
    bool
        whether more calls to tqmap is necessary

    """

def tqmodl(indexp: int) -> SolutionModel:
    """
    Get the solution model name for the specified phase

    Parameters
    ----------
    indexp
        zero-based phase index

    Returns
    -------
    SolutionModel
        solution model name
    """

def tqnexc(indexp: int) -> int:
    """
    Get the number of ternary exceptions of a phase.

    Parameters
    ----------
    indexp
        zero-based phase index

    Returns
    -------
    int
        number of ternary oexceptions
    """

def tqnfct() -> int:
    """
    Get the total number of functions.

    Returns
    -------
    int
        total number of functions

    Note
    ----
    You can only use this function if the ChemSage.dat contains
    functions, which is available when exporting from FactSage 8.1+
    """

def tqnolc(indexp: int, indexl: int) -> int:
    """
    Get the number of sublattice constituents of the specified sublattice.

    Parameters
    ----------
    indexp
        zero-based phase index
    indexl
        zero-based sublattice index

    Returns
    -------
    int
        number of sublattice constituents
    """

def tqnop() -> int:
    """
    Get the total number of phases.

    Returns
    -------
    int
        number of phases
    """

def tqnopc(indexp: int) -> int:
    """
    Get the total number of constituents in the specified phase.

    Parameters
    ----------
    indexp
        zero-based phase index

    Returns
    -------
    int
        number of phase constituents
    """

def tqnosc() -> int:
    """
    Get the total number of system components.

    Returns
    -------
    int
        number of system components
    """

def tqnosl(indexp: int) -> int:
    """
    Get the number of sublattices of the specified phase.

    Parameters
    ----------
    indexp
        zero-based phase index

    Returns
    -------
    int
        number of sublattices
    """

def tqnpar(indexp: int, option: InteractionVariable) -> int:
    """
    Get the number of excess/interaction parameters of a phase.

    Parameters
    ----------
    indexp
        zero-based phase index
    option
        ('G') Gibbs energy or ('V') volume

    Returns
    -------
    int
        number of excess parameters
    """

def tqopen(file_path: str | Path, unit: int) -> None:
    """
    Open a file for reading or writing by ChemApp.

    The ``file_path`` is instantiated as a ``pathlib.Path`` object.

    Parameters
    ----------
    file_path
        path of the file to open
    unit
        unit number under which to open the file
    """

def tqopna(file_path: str | Path, unit: int) -> None:
    """
    Open an ASCII thermochemical data file for reading by ChemApp.

    The ``file_path`` is instantiated as a ``pathlib.Path`` object.

    Parameters
    ----------
    file_path
        path of the file to open
    unit
        unit number under which to open the file
    """

def tqopnt(file_path: str | Path, unit: int) -> None:
    """
    Open a transparent thermochemical data file for reading by ChemApp.

    The ``file_path`` is instantiated as a ``pathlib.Path`` object.

    Parameters
    ----------
    file_path
        path of the file to open
    unit
        unit number under which to open the file
    """

def tqpcis(indexp: int, indexc: int) -> bool:
    """
    Check if the specified phase constituent is permitted as an incoming
    species.

    Parameters
    ----------
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index

    Returns
    -------
    bool
        permittance
    """

def tqrcst() -> None:
    """
    Read a thermodynamic data-file in transparent format.
    """

def tqremc(numcon: int) -> None:
    """
    Remove a condition from the equilibrium calculation.

    Parameters
    ----------
    numcon
        condition number
    """

def tqrfil() -> None:
    """
    Read a thermodynamic data-file in ASCII format.
    """

def tqsetc(option: ConditionVariable, indexp: int, indexc: int, val: float) -> int:
    """
    Set a condition for the equilibrium calculation.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.ConditionVariable


    Parameters
    ----------
    option
        option indicating the variable to set
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent or system component index
    val
        condition value

    Returns
    -------
    int
        condition number


    """

def tqshow() -> None:
    """
    Display the present settings.

    tqshow displays information about the currently set conditions and streams,
    and is useful for debugging. Its output is stored on the unit/file
    associated with LIST.

    It does not, effectively, print the output.

    """

def tqsize() -> Tuple[int, int, int, int, int, int, int, int, int, int, int]:
    """
    Get ChemApp's internal array dimensions.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/basic/index.html#chemapp.basic.tqsize


    Returns
    -------
    Tuple[int, int, int, int, int, int, int, int, int, int, int]
        (na, nb, nc, nd, ne, nf, ng, nh, ni, nj, nk)

    """

def tqstca(idents: str, indexp: int, indexc: int, val: float) -> None:
    """
    Set a phase constituent amount for the specified stream.

    Parameters
    ----------
    idents
        stream name
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index
    val
        phase constituent amount
    """

def tqstec(option: ConditionVariable, indexp: int, val: float) -> None:
    """
    Set an invariant state variable for the equilibrium calculation when the
    input conditions are defined by streams.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.ConditionVariable


    Parameters
    ----------
    option
        option indicating the variable to set
    indexp
        zero-based phase index
    val
        condition value

    """

def tqstpc(indexp: int, indexc: int) -> Tuple[List[float], float]:
    """
    Get the stoichiometry and molecular mass of the specified phase constituent.


    Parameters
    ----------
    indexp
        zero-based phase index
    indexc
        zero-based phase constituent index

    Returns
    -------
    Tuple[List[float], float]
        Stoichiometric coefficients for all system components, and the molar
        mass of the phase constituent


    """

def tqstrm(idents: str) -> None:
    """
    Remove a stream.

    Parameters
    ----------
    idents
        stream name
    """

def tqstsc(indexs: int) -> Tuple[List[float], float]:
    """
    Get the stoichiometry and molecular mass of a system component.


    Parameters
    ----------
    indexs
        zero-based system component index

    Returns
    -------
    Tuple[List[float], float]
        Stoichiometric coefficients of the system component, and the molar
        mass of the system component

    """

def tqsttp(idents: str, T: float, P: float) -> None:
    """
    Define a stream and set its temperature and pressure.

    Parameters
    ----------
    idents
        stream name
    T
        temperature of the stream
    P
        pressure of the stream


    The input variable idents can have a maximum length of 150 characters. VALS
    must be a list of 2 real values, containing the temperature and pressure of
    the stream. These are entered with the default unit, or a unit set using
    :py:meth:`~chemapp.basic.tqcsu`. A stream is a means for transferring
    non-reacted matter to a reaction zone. It has constant temperature and
    pressure, and contains one or more phases of fixed composition. Stream input
    is required whenever differences in extensive properties between the initial
    and the equilibrium state are desired; for example, in calculations of the
    heat evolving from a combustion process, or of the entropy change caused by
    a phase transition. The maximum number of streams which are defined must not
    be greater than the maximum number of constituents permitted in the ChemApp
    version currently used (see :py:meth:`~chemapp.basic.tqsize`, parameter 'NA').
    """

def tqstxp(idents: str, option: StreamVariable) -> float:
    """
    Get the calculated thermodynamic properties of a stream.

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/core/index.html#chemapp.core.StreamVariable


    Parameters
    ----------
    idents
        name of the stream
    option
        option identifying the thermodynamic quantity

    Returns
    -------
    float
        thermodynamic quantity

    """

def tqused() -> Tuple[int, int, int, int, int, int, int, int, int, int, int]:
    """
    Get the dimensions of the currently loaded thermochemical system.

    Returns
    -------
    Tuple[int, int, int, int, int, int, int, int, int, int, int]
        (na, nb, nc, nd, ne, nf, ng, nh, ni, nj, nk)

    More information:

    https://python.gtt-technologies.de/doc/chemapp/autoapi/chemapp/basic/index.html#chemapp.basic.tqused


    """

def tqvers() -> int:
    """
    Get the ChemApp version number.

    Returns
    -------
    int
        ChemApp version number
    """

def tqwasc(file: str | Path) -> None:
    """
    Write a thermodynamic data-file in ASCII format.

    Parameters
    ----------
    file
        name of the data file
    """

def tqwstr(option: IoOption, text: str) -> None:
    """
    Write a character string to the Fortran units associated with 'LIST' and
    'ERROR'.

    Parameters
    ----------
    option
        either 'LIST' or 'ERROR'
    text
        a character string of arbitrary length (be reasonable! ;)
    """
