# encoding: utf-8

"""
This module provides base infrastructure used in the ChemApp for Python package.

The following abbreviations are used in the interface to name functions,
parameters, etc.:

+--------------+--------------------+--------------+---------------------+
| Abbreviation | Term               | Abbreviation | Quantity            |
+==============+====================+==============+=====================+
| eq           | equilibrium        | A            | amount              |
+--------------+--------------------+--------------+---------------------+
| cf           | chemical formula   | Cp           | heat capacity       |
+--------------+--------------------+--------------+---------------------+
| mm           | molar mass         | E            | energy              | 
+--------------+--------------------+--------------+---------------------+
| ph           | phase              | G            | Gibbs energy        |
+--------------+--------------------+--------------+---------------------+
| phs          | phases             | H            | enthalpy            |
+--------------+--------------------+--------------+---------------------+
| pc           | phase constituent  | IA           | incoming amount     |
+--------------+--------------------+--------------+---------------------+
| pcs          | phase constituents | MU           | chemical potential  |
+--------------+--------------------+--------------+---------------------+
| sc           | system component   | P            | pressure            |
+--------------+--------------------+--------------+---------------------+
| scs          | system components  | S            | entropy             |
+--------------+--------------------+--------------+---------------------+
| st           | stream             | T            | temperature         |
+--------------+--------------------+--------------+---------------------+
| sts          | streams            | V            | volume              |
+--------------+--------------------+--------------+---------------------+
| tg           | target             | VT           | total volume        |
+--------------+--------------------+--------------+---------------------+
| tx           | transition         | X            | fraction            |
+--------------+--------------------+--------------+---------------------+
| txs          | transitions        | XT           | mole fraction       |
+--------------+--------------------+--------------+---------------------+
|              |                    | y            | mass fraction       |
+--------------+--------------------+--------------+---------------------+

"""

from __future__ import annotations
import csv as csv
import enum
import gc as gc
from jinja2.environment import Environment
from jinja2.loaders import FileSystemLoader
import sys as sys
import time as time
import traceback as traceback
import typing

from typing import FrozenSet, List, Dict, Tuple

__all__ = [
    "AmountUnit",
    "ChemAppError",
    "ChemAppInfo",
    "ChemAppLightError",
    "ChemAppPyError",
    "ChemicalFormulaIncomingAmountState",
    "Composition",
    "ConditionVariable",
    "ConfigurationOption",
    "ConvertableObject",
    "DataFormat",
    "EnergyUnit",
    "Environment",
    "EquilibriumCalculationResult",
    "FileSystemLoader",
    "FunctionData",
    "FunctionSumVariable",
    "IncomingAmountState",
    "InteractionVariable",
    "IoOption",
    "LimitVariable",
    "PhaseConfig",
    "PhaseConstituentConfig",
    "PhaseConstituentData",
    "PhaseConstituentIncomingAmountState",
    "PhaseConstituentState",
    "PhaseConstituentVariable",
    "PhaseMapVariable",
    "PhaseMatterState",
    "PhaseState",
    "PressureUnit",
    "Quantity",
    "ResultVariable",
    "SolutionModel",
    "Status",
    "StreamCalculationResult",
    "StreamState",
    "StreamVariable",
    "SystemComponentConfig",
    "SystemComponentIncomingAmountState",
    "SystemComponentState",
    "TargetVariable",
    "TemperatureUnit",
    "ThermochemicalSystemConfig",
    "VolumeUnit",
    "find_state",
    "switch_components",
]

class AmountUnit(enum.Enum):
    """
    Available amount units.

    +--------+
    | Amount |
    +========+
    | mol    |
    +--------+
    | gram   |
    +--------+
    | kg     |
    +--------+
    | tonne  |
    +--------+
    """

    def __init__(self):
        """This class is not meant to be initialized."""

class EnergyUnit(enum.Enum):
    """
    Available energy units.

    +--------+
    | Energy |
    +========+
    | J      |
    +--------+
    | cal    |
    +--------+
    | Btu    |
    +--------+
    | kWh    |
    +--------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class VolumeUnit(enum.Enum):
    """
    Available volume units.


    +--------+
    | Volume |
    +========+
    | dm3    |
    +--------+
    | cm3    |
    +--------+
    | m3     |
    +--------+
    | ft3    |
    +--------+
    | in3    |
    +--------+
    """

    def __init__(self):
        """This class is not meant to be initialized."""

class PressureUnit(enum.Enum):
    """
    Available pressure units.

    +----------+
    | Option   |
    +==========+
    |  bar     |
    +----------+
    | atm      |
    +----------+
    | Pa       |
    +----------+
    | kPa      |
    +----------+
    | psi      |
    +----------+
    | torr     |
    +----------+
    """

    def __init__(self):
        """This class is not meant to be initialized."""

class TemperatureUnit(enum.Enum):
    """
    Available temperature units.

    +-------------+
    | Temperature |
    +=============+
    | K           |
    +-------------+
    | C           |
    +-------------+
    | F           |
    +-------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class Quantity(enum.Enum):
    """
    Quantities available for setting units.

    +-------------+-------+
    | Quantity    | Unit  |
    +=============+=======+
    | Pressure    | bar   |
    |             +-------+
    |             | atm   |
    |             +-------+
    |             | Pa    |
    |             +-------+
    |             | kPa   |
    |             +-------+
    |             | psi   |
    |             +-------+
    |             | torr  |
    +-------------+-------+
    | Volume      | dm3   |
    |             +-------+
    |             | cm3   |
    |             +-------+
    |             | m3    |
    |             +-------+
    |             | ft3   |
    |             +-------+
    |             | in3   |
    +-------------+-------+
    | Temperature | K     |
    |             +-------+
    |             | C     |
    |             +-------+
    |             | F     |
    +-------------+-------+
    | Energy      | J     |
    |             +-------+
    |             | cal   |
    |             +-------+
    |             | Btu   |
    |             +-------+
    |             | kWh   |
    +-------------+-------+
    | Amount      | mol   |
    |             +-------+
    |             | gram  |
    |             +-------+
    |             | kg    |
    |             +-------+
    |             | tonne |
    +-------------+-------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class ConditionVariable(enum.Enum):
    """

    Available variables for setting conditions in equilibrium calculations.

    .. tabularcolumns:: |p{1cm}|p{3cm}|p{8cm}|

    +--------+--------------------+-------------------------------------------------------------------------------------+
    | Value  | Variable           | Description                                                                         |
    +========+====================+=====================================================================================+
    | P      | Total pressure     | Last entered (pressure or volume) takes precedence. Default value is 1 bar.         |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | VT     | Total volume       | Last entered (pressure or volume) takes precedence. Note that the use of this       |
    |        |                    | option to perform calculations at constant volume is discouraged, see Appendix B.4  |
    |        |                    | for details.                                                                        |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | T      | Temperature        | Default value is 1000 K.                                                            |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | A      | Amount of phase    | VAL >=0 defines a formation target calculation; VAL <0 a precipitation target       |
    |        |                    | calculation for a mixture phase. In a formation target calculation, the target      |
    |        |                    | variable is varied until unit activity and zero amount of the target phase is       |
    |        |                    | achieved. A precipitation target is defined as one where the target phase is the    |
    |        |                    | only one formed at equilibrium, and the second most stable phase has unit activity. |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | IA     | Incoming amount    | Valid for phase constituents and system components. Not permitted for TQSTEC.       |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | MU     | Chemical potential | Valid only for phase constituents. Not permitted for TQSTEC.                        |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | AC     | Relative activity, | Valid only for phase constituents. Not permitted for TQSTEC.                        |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | CP,    | Heat capacity,     | Valid for the entire system or for a phase. Extensive properties are either         |
    | H,     | Enthalpy,          | calculated for the equilibrium phases and are then dependent on the reference state |
    | S,     | Entropy,           | chosen in the data-file, or for the chemical reaction when streams are defined, in  |
    | G,     | Gibbs energy,      | which case they are relative to the state defined by the stream conditions.         |
    | V,     | Volume             |                                                                                     |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | AT     | Relative activity  | Relative activity of a phase constituent as target.                                 |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | XT     | Mole fraction      | Mole fraction of a phase constituent as target.                                     |
    +--------+--------------------+-------------------------------------------------------------------------------------+
    | XPT    | Mole fraction      | Mole fraction of a system component in a phase as target.                           |
    +--------+--------------------+-------------------------------------------------------------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class InteractionVariable(enum.Enum):
    """
    Enum to specific the type of interaction parameter to extract.

    +----------+-----------------------------------------+
    | Option   | Meaning                                 |
    +==========+=========================================+
    | G        | Gibbs energy function excess parameters |
    +----------+-----------------------------------------+
    | M        | Magnetic interaction parameters         |
    +----------+-----------------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class LimitVariable(enum.Enum):
    """
    Limit variables that can be set during target calculations.

    .. tabularcolumns:: |p{1cm}|p{3cm}|p{2cm}|

    +--------+------------------------+---------------+
    | Option | Variable Limit         | Default Value |
    +========+========================+===============+
    | PLOW   | Low pressure limit     | 1E-50 bar     |
    +--------+------------------------+---------------+
    | PHIGH  | High pressure limit    | 1E7 bar       |
    +--------+------------------------+---------------+
    | VLOW   | Low volume limit       | 1E-7 dm3      |
    +--------+------------------------+---------------+
    | VHIGH  | High volume limit      | 1E50 dm3      |
    +--------+------------------------+---------------+
    | TLOW   | Low temperature limit  | 298.15 K      |
    +--------+------------------------+---------------+
    | THIGH  | High temperature limit | 6000 K        |
    +--------+------------------------+---------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class IoOption(enum.Enum):
    """
    Input/output options used by :py:meth:`~chemapp.basic.tqgio` and
    :py:meth:`~chemapp.basic.tqcio`.

    .. tabularcolumns:: |p{3cm}|p{6cm}|p{3cm}|

    +----------+--------------------------------------------------------------+---------------+
    | Option   | Meaning                                                      | Default value |
    +==========+==============================================================+===============+
    | FILE     | Unit from which the thermodynamic data-file is read          | 10            |
    +----------+--------------------------------------------------------------+---------------+
    | ERROR    | Unit to which error messages are directed                    | 6             |
    +----------+--------------------------------------------------------------+---------------+
    | LIST     | Unit to which optional equilibrium output lists are directed | 6             |
    +----------+--------------------------------------------------------------+---------------+
    | LANGUAGE | Language used for error messages                             | 1             |
    |          | (1=English, 2=German, 3=French, 4=Swedish, 5=Finnish)        |               |
    +----------+--------------------------------------------------------------+---------------+


    """

    def __init__(self):
        """This class is not meant to be initialized."""

class ConfigurationOption(enum.Enum):
    """Configuration options for ChemApp.

    Parameters
    ----------
    E
        The electron configuration option.
    """

    def __init__(self):
        """This class is not meant to be initialized."""

class ChemAppError(ChemAppPyError):
    """
    Errors generated by the ChemApp API's standard error reporting mechanism.

    It is possible to acquire the ChemApp error number via the `errno`
    property. If this is not applicable, it will return 0.

    """

    @property
    def errno(self) -> int:
        """
        <int> The error number of ChemApp, if applicable. Defaults to 0.
        """

class ChemAppInfo(Object):
    """
    This class provides information about ChemApp.
    """

    @property
    def array_size_descriptions(self):
        """
        Get the array size descriptions of ChemApp's memory setup.
        """

    @property
    def array_sizes_max(self):
        """
        Get the maximum array sizes of ChemApp.
        """

    @property
    def array_sizes_used(self):
        """
        Get the currently used array size descriptions of ChemApp.
        """

    @property
    def copyright(self):
        """
        Get the copyright message of ChemApp.
        """

    @property
    def dongle_id(self):
        """
        Get the dongle ID.
        """

    @property
    def dongle_type(self):
        """
        Get the dongle type.
        """

    @property
    def expiration_date(self):
        """
        Get the ChemApp expiration date.
        """

    @property
    def is_light(self):
        """
        Get if the light version of ChemApp is used.
        """

    @property
    def program_id(self):
        """
        Get the ChemApp program id.
        """

    @property
    def user_id(self):
        """
        Get the user id.
        """

    @property
    def user_name(self):
        """
        Get the user name.
        """

    @property
    def version(self):
        """
        Get the ChemApp version.
        """

class ChemAppLightError(ChemAppPyError):
    """
    Errors related to ChemApp light.

    """

class ChemAppPyError(Exception):
    """
    Base class for all errors related to the ChemApp python package.

    """

class ChemicalFormulaIncomingAmountState(IncomingAmountState):
    """
    This class represents the state of a chemical formula incoming amount in
    an equilibrium calculation result.

    Parameters
    ----------
    cf : str
        chemical formula
    A : float
        amount value
    """

    @property
    def A(self) -> float:
        """
        Amount.

        Returns
        -------
        float
            Amount

        """

    @property
    def cf(self) -> str:
        """
        Chemical formula name.

        Returns
        -------
        str
            chemical formula

        """

    @property
    def scs(self) -> List[str]:
        """
        System components present in the chemical formula.

        Returns
        -------
        List[str]
            System components in the chemical formula
        """

    @property
    def scs_A(self) -> Dict[str, float]:
        """
        System components and amounts in the chemical formula.

        Returns
        -------
        Dict[str, float]
            calculated system component amounts

        """

class Composition:
    """
    Represents a Composition, which is essentially a {element:amount} mapping

    It can be parsed from a string representation of a composition. Some
    examples: "Fe2O3", "Sr(OH)3", "H2O", "LiPO4".

    It can also be created as a copy of another `Composition` class, or from a
    dictionary of elements:amounts mappings. For consistency reasons, the order
    of the elements is preserved as entered.

    Parameters
    ----------
    *args: str | Composition | Dict[str,float]
        The composition to create.

    """

    amount_tolerance: typing.ClassVar[float] = 1e-06
    def to_dict(self, reduced=False) -> Dict[str, float]:
        """
        Get the composition as dictionary {element:amount}.

        Parameters
        ----------
        reduced
            (default: False) Whether the smallest applicable scaling factor is
            applied.

        Examples
        --------
        >>> Composition("Fe4O6").to_dict()
        {"Fe": 4.0, "O": 6.0}

        >>> Composition("Fe4O6").to_dict(reduced=True)
        {"Fe": 2.0, "O": 3.0}

        Returns
        -------
        Dict[str, float]
            Dictionary of {element:amount} of the composition
        """

    @property
    def elements(self) -> FrozenSet[str]:
        """
        The elements the composition contains.

        Returns
        -------
        FrozenSet[str]
            element set
        """

    @property
    def formula(self) -> str:
        """
        String representation of the composition.

        This gives the formula after internal parsing. It may be differently
        formatted compared to the input formula.

        Examples
        --------
        >>> Composition("H20").formula
        'H20'

        >>> Composition("Cs(OH)3").formula
        'Cs1O3H3'

        Returns
        -------
        str
            printable composition
        """

    @property
    def num_atoms(self) -> float:
        """
        Get the number of atoms in the formula.

        Returns
        -------
        float
            sum of all constituent atom amounts

        """

    @property
    def pretty_formula(self) -> str:
        """
        Get the pretty formula of the composition

        Elements sorted by Pauling order. No grouping of functional groups. If
        an element has the amount equal to 1, that number is omitted.

        Returns
        -------
        str
            printable composition
        """

    @property
    def pretty_reduced_formula(self) -> str:
        """
        Get the pretty reduced formula of the composition

        Elements sorted by Pauling order. No grouping of functional groups.
        Smallest common denominator applied to the element composition.

        Returns
        -------
        str
            printable composition

        """

    @property
    def reduced_formula(self) -> str:
        """
        Get the reduced formula of the composition

        Smallest common denominator applied to the element composition.

        Returns
        -------
        str
            printable composition
        """

    @property
    def unordered(self) -> Dict[str, float]:
        """
        Get the as entered order of elements.


        Returns
        -------
        Dict[str, float]
            elemental composition
        """

class EquilibriumCalculationResult(ConvertableObject):
    """
    An object of this class stores all the results of a single equilbrium
    calculation result, regardless of the type of calculation (normal, stream,
    phase map).

    Parameters
    ----------
    units : Dict[str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit]
        units as returned by Units.get()
    ts_md5 : str
        md5 checksum of the thermochemical data file that the result is based on
    t_start : float
        calculation start timestamp
    t_end : float
        calculation end timestamp
    dt :  float
        calculation time

    """

    def create_gas_stream(self, name: str, include_zeros: bool = False) -> StreamState:
        """
        Create a stream state object from this result object including only
        phases that are gasses.

        Parameters
        ----------
        name
            stream name
        include_zeros
            include zero values (default: False)

        Returns
        -------
        StreamState
            new stream object

        """

    def create_liquid_stream(
        self, name: str, include_zeros: bool = False
    ) -> StreamState:
        """
        Create a stream state object from this result object including only
        phases that are liquid.

        Parameters
        ----------
        name
            stream name
        include_zeros
            include zero values (default: False)

        Returns
        -------
        StreamState
            new stream object
        """

    def create_solid_stream(
        self, name: str, include_zeros: bool = False
    ) -> StreamState:
        """
        Create a stream state object from this result object including only
        phases that are solid.

        Parameters
        ----------
        name
            stream name
        include_zeros
            include zero values (default: False)

        Returns
        -------
        StreamState
            new stream object

        """

    def create_stream(
        self,
        name: str,
        include_zeros: bool = False,
        phs_include: List[str] | None = None,
        phs_exclude: List[str] | None = None,
        els_include: List[str] | None = None,
        els_exclude: List[str] | None = None,
        phsconfigs: ThermochemicalSystemConfig | None = None,
        el_threshold: float = 0.0,
    ) -> StreamState:
        """
        Create a stream state object from this result object.

        Either `phs_include` _or_ `phs_exclude` can be defined. Equally, only
        either `els_include` _or_ `els_exclude` can be used. If the latter is
        employed, the parameter `phsconfig` needs to be given, too. It can be
        acquired through `ThermochemicalSystem.get_config_phs()`.

        Parameters
        ----------
        name
            stream name
        include_zeros
            include zero values
        phs_include
            phases to include
        phs_exclude
            phases to ignore
        els_include
            elements to include
        els_exclude
            elements to ignore
        phsconfigs
            phase configurations, as returned by `ThermochemicalSystem.get_config_phs()`
        el_threshold
            minimum threshold value to be considered (default: 0.0)

        Returns
        -------
        StreamState
            new stream object
        """

    def create_stream_by_state(
        self, state: PhaseMatterState, name: str, include_zeros: bool = False
    ) -> StreamState:
        """
        Create a stream state object from this result object including only
        phases that match the 'state'.

        Parameters
        ----------
        state
            phase state
        name
            stream name
        include_zeros
            include zero values (default: False)

        Returns
        -------
        StreamState
            new stream object

        """

    def get_eq_phs(self, A: float = 0.0) -> List[PhaseState]:
        """
        Get the list of stable phases from this result object.

        The criteria for stability is phase activity to be equal to 1, and the
        amount being larger than the given threshold `A`

        Parameters
        ----------
        A
            minimum threshold for a stable phase amount (default: 0.0)

        Returns
        -------
        List[PhaseState]
            list of references to phase state objects

        """

    @property
    def IAs(self) -> List[IncomingAmountState]:
        """
        The specified or calculated incoming amounts.

        Returns
        -------
        List[IncomingAmountState]
            incoming amounts
        """

    @property
    def dt(self) -> float:
        """
        Calculation duration.

        Returns
        -------
        float
            duration, seconds.
        """

    @property
    def id(self) -> int:
        """
        The result's ID.

        Returns
        -------
        int
            result id

        """

    @property
    def labels(self) -> Dict[str, str]:
        """
        Labels that annotate and classify the calculation result.

        Returns
        -------
        Dict[str,str]
            Labels
        """

    @property
    def phs(self) -> Dict[str, PhaseState]:
        """
        The equilibrium phase states.

        Returns
        -------
        Dict[str, PhaseState]
            phase states
        """

    @property
    def scs(self) -> Dict[str, SystemComponentState]:
        """
        The equilibrium system component states.

        Returns
        -------
        Dict[str, PhaseState]
            system component states
        """

    @property
    def t_end(self) -> float:
        """
        Calculation end time.

        Returns
        -------
        float
            end time
        """

    @property
    def t_start(self) -> float:
        """
        Calculation start time.

        Returns
        -------
        float
            start time
        """

    @property
    def ts_md5(self) -> str:
        """
        The md5 checksum of the thermochemical data file that the result is
        based on.

        Returns
        -------
        float
            md5 hash

        """

    @property
    def units(
        self,
    ) -> Dict[
        str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit
    ]:
        """
        The units used for the calculation.

        Returns
        -------
        Dict[str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit]
            units

        """

class StreamCalculationResult(EquilibriumCalculationResult):
    """See EquilibriumCalculationResult"""

    def streams(self) -> Dict[str, StreamState]:
        """Get the streams that are input to the equilibrium calculation.


        Returns
        -------
        Dict[StreamState]
            Streams that were input to the equilibrium calculation.

        """

class FunctionData(enum.Enum):
    """
    Available data items that can be requested for functions.


    .. tabularcolumns:: |p{2cm}|p{8cm}|

    +--------------+---------------------------+
    | FunctionData | Description               |
    +==============+===========================+
    | H            | Enthalpy                  |
    +--------------+---------------------------+
    | S            | Entropy                   |
    +--------------+---------------------------+
    | Cp           | Cp function coefficients  |
    +--------------+---------------------------+
    | Tt           | Temperature of transition |
    +--------------+---------------------------+
    | Ht           | Heat of transition        |
    +--------------+---------------------------+
    | V            | Volume                    |
    +--------------+---------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class FunctionSumVariable(enum.Enum):
    """
    Available function sum data for ChemSage.dat with functions.


    +----------+----------------------------------+
    | Option   | Meaning                          |
    +==========+==================================+
    | G        | Gibbs energy function parameters |
    +----------+----------------------------------+
    | V        | Volumetric expansion parameters  |
    +----------+----------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class IncomingAmountState(Object):
    """
    This abstract base class represents the state of an incoming amount in
    an equilibrium calculation result.

    Parameters
    ----------
    A : float
        amount value
    """

    @property
    def A(self) -> float:
        """
        Amount.

        Returns
        -------
        float
            amount
        """

class DataFormat(enum.Enum):
    """
    This enumeration contains a list of data formats that ChemApp for Python objects
    can be converted to and from. The formats are as follows:

    * ``indexedPrimitiveDoc``

      A document built of primitive Python data types. The root type is a
      ``dict``, and nested containers are stored as lists so that they are
      indexed.

    * ``mongoDbIndexedPrimitiveDoc``

      The same as ``indexedPrimitiveDoc`` with one exception. Dots (``.``) in
      element names are replaced with underscores to prevent query problems in
      MongoDB.
    """

    def __init__(self):
        """This class is not meant to be initialized."""

class PhaseConfig(ObjectDescriptor):
    """

    This class represents the configuration of a phase as read from a
    thermochemical data file.

    Parameters
    ----------
    index : int
        phase one-based index
    name : str
        phase name
    model : SolutionModel
        phase's solution model
    pcs : List[PhaseConstituentConfig]
        phase constituent configuration objects
    alt_name : str
        phase's alternative name

    """

    @property
    def alt_name(self) -> str:
        """
        The phase's alternative name.

        Returns
        -------
        str
            name
        """

    @property
    def model(self) -> SolutionModel:
        """
        The phase's solution model.

        Returns
        -------
        SolutionModel
            phase model
        """

    @property
    def pcs(self) -> Dict[str, PhaseConstituentState]:
        """
        The equilibrium phase constituent states.

        Returns
        -------
        Dict[str, PhaseConstituentState]
            phase constituent states
        """

class PhaseConstituentConfig(ObjectDescriptor):
    """
    This class represents the configuration of a phase constituent as read from
    a file.

    Parameters
    ----------
    index : int
        phase constituent one-based index
    name : str
        phase constituent name
    mm : float
        phase constituent molar mass
    stoich : Dict[str,float]
        phase constituent stoichiometry
    y_scs :  Dict[str,float]
        system component mass fractions
    """

    @property
    def mm(self) -> float:
        """
        The phase constituent's molar mass.

        Returns
        -------
        float
            molar mass
        """

    @property
    def stoich(self) -> Dict[str, float]:
        """
        The phase constituent's stoichiometry.

        Returns
        -------
        Dict[str,float]
            stoichiometry
        """

    @property
    def y_scs(self) -> Dict[str, float]:
        """
        The mass fraction of system components in the constituent.

        Returns
        -------
        Dict[str,float]
            mass fractions
        """

class PhaseConstituentData(enum.Enum):
    """
    Available data items that can be requested for phase constituents.


    +--------+-----------------------------------------------------------------------------+
    | OPTION | Variable                                                                    |
    +========+=============================================================================+
    | H      | Enthalpy/J at 298.15K                                                       |
    +--------+-----------------------------------------------------------------------------+
    | S      | Entropy/J*K-1 at 298.15K                                                    |
    +--------+-----------------------------------------------------------------------------+
    | Cp     | Heat capacity expression/J.K-1 or Helgeson terms for                        |
    |        | the models PIHZ, HELZ, HTSZ, HTWZ or HTDZ                                   |
    +--------+-----------------------------------------------------------------------------+
    | V      | Molar volume data in input order of a condensed phase constituent           |
    +--------+-----------------------------------------------------------------------------+
    | Gc     | Critical properties in input order of a gas phase constituent               |
    +--------+-----------------------------------------------------------------------------+
    | Ht     | Transformation enthalpy/J at the upper temperature limit for a Cp range     |
    +--------+-----------------------------------------------------------------------------+
    | M      | Curie/Neel temperature and average magnetic moment per atom for a magnetic  |
    |        | phase constituent                                                           |
    +--------+-----------------------------------------------------------------------------+
    | Tt     | Upper temperature limit/K for a Cp range                                    |
    +--------+-----------------------------------------------------------------------------+
    | Ch     | Charge of an aqueous phase constituent                                      |
    +--------+-----------------------------------------------------------------------------+


    """

    def __init__(self):
        """This class is not meant to be initialized."""

class PhaseConstituentIncomingAmountState(IncomingAmountState):
    """
    This class represents the state of an phase constituent incoming amount in
    an equilibrium calculation result.

    Parameters
    ----------
    ph : str
        phase name
    pc : str
        phase constituent name
    A : float
        amount value

    """

    @property
    def pc(self) -> str:
        """
        Phase constituent name.

        Returns
        -------
        str
            constituent name

        """

    @property
    def ph(self) -> str:
        """
        Phase name.

        Returns
        -------
        str
            phase name


        """

class PhaseConstituentState(ObjectDescriptor):
    """
    This class represents the state of a phase constituent in an equilibrium
    calculation result.

    Parameters
    ----------
    index : int
        phase constituent one-based index
    name : str
        phase constituent name
    status : Status
        phase constituent status

    """

    @property
    def status(self) -> Status:
        """
        The phase constituent's status.

        Returns
        -------
        Status
            phase constituent status

        """

class PhaseConstituentVariable(enum.Enum):
    """
    Available variables for extracting thermodynamic properties of phase
    constituents.

    +-------+--------------------------------------------------------------+
    | Value | Description                                                  |
    +=======+==============================================================+
    | CP    | Heat capacity                                                |
    +-------+--------------------------------------------------------------+
    | G     | Gibbs energy                                                 |
    +-------+--------------------------------------------------------------+
    | H     | Enthalpy                                                     |
    +-------+--------------------------------------------------------------+
    | S     | Entropy                                                      |
    +-------+--------------------------------------------------------------+
    | T     | Temperature (used to retrieve the upper limit of the current |
    |       | temperature interval for a given phase constituent or a      |
    |       | stoichiometric condensed phase)                              |
    |       |                                                              |
    +-------+--------------------------------------------------------------+
    | V     | Volume                                                       |
    +-------+--------------------------------------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class PhaseMapVariable(enum.Enum):
    """
    Variables that can be used to adjust conditions as part of phase map
    calculations.

    +-------------+-----------------+-------------------------------------------------------------------+
    | Option      | Variable        |  Comment                                                          |
    +=============+=================+===================================================================+
    | PF /        | Total pressure  | Pressure is the search variable. The upper and lower bounds of    |
    | PN          |                 | the pressure search interval are in VALS(1) and VALS(2). Use      |
    |             |                 | option 'PF' for the first call to :py:meth:`~chemapp.basic.tqmap`,|
    |             |                 | and 'PN' for all subsequent ones.                                 |
    |             |                 |                                                                   |
    +-------------+-----------------+-------------------------------------------------------------------+
    | TF /        | Temperature     | Temperature is the search variable. The upper and lower bounds    |
    | TN          |                 | of the temperature search interval are in VALS(1) and VALS(2).    |
    |             |                 | Use option 'TF' for the first call to                             |
    |             |                 | :py:meth:`~chemapp.basic.tqmap`, and 'TN' for all subsequent      |
    |             |                 | ones.                                                             |
    +-------------+-----------------+-------------------------------------------------------------------+
    | IA0 /       | Incoming amount | Incoming amount is the search variable. The upper and lower       |
    | IAF /       |                 | bounds of the composition search interval are in VALS(1) and      |
    | IAN         |                 | VALS(2). If incoming amounts for more than one substance need     |
    |             |                 | to be defined, all except the last one have to be called using    |
    |             |                 | 'IA0' as option to :py:meth:`~chemapp.basic.tqmap`. Once 'IAF' is |
    |             |                 | passed for the last incoming amount, the first calculation is     |
    |             |                 | made. Use option 'IAN' for all subsequent ones.                   |
    |             |                 |                                                                   |
    +-------------+-----------------+-------------------------------------------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class PhaseMatterState(enum.Enum):
    """
    Aggregate state of a phase.

    +--------+-----------------+
    | Option | Matter of State |
    +========+=================+
    | SOLID  | Solid           |
    +--------+-----------------+
    | LIQUID | Liquid          |
    +--------+-----------------+
    | GAS    | Gas             |
    +--------+-----------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class PhaseState(ObjectDescriptor):
    """
    This class represents the state of a phase in an equilibrium calculation
    result.

    Parameters
    ----------
    index : int
        phase one-based index
    name : str
        phase name
    status : Status
        phase status

    """

    @property
    def pcs(self) -> Dict[str, PhaseConstituentState]:
        """
        The equilibrium phase constituent states.

        Returns
        -------
        Dict[PhaseConstituentState]
            phase constituent states
        """

    @property
    def scs(self) -> Dict[str, SystemComponentState]:
        """
        The equilibrium system component states.

        Returns
        -------
        Dict[SystemComponentState]
            system component states
        """

    @property
    def status(self) -> Status:
        """
        The phase's status.

        Returns
        -------
        Status
            phase status
        """

class ResultVariable(enum.Enum):
    """
    Result variables available after equilibrium calculations.

    .. tabularcolumns:: |p{1cm}|p{3cm}|p{10cm}|

    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | Option | Variable                    | Comment                                                                                           |
    +========+=============================+===================================================================================================+
    | P      | Total pressure              | ---                                                                                               |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | VT     | Total volume                | ---                                                                                               |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | T      | Temperature                 | ---                                                                                               |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | A      | Equilibrium amount          | ---                                                                                               |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | IA     | Incoming amount             | Not valid for phases or for the entire system                                                     |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | MU,    | Chemical potential,         | Relative values for phases and constituents; absolute values for system components;               |
    | AC     | Activity/fugacity           | fugacities in current pressure unit for gas phase constituents.                                   |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | pH,    | pH,                         | The calculation of pH and Eh/V requires that the constituents H+ in the aqueous phase and H2 in   |
    | Eh     | Eh/V                        | the gas phase be present in the thermochemical data-file. If pH and Eh/V cannot be calculated,    |
    |        |                             | theoutput VAL=0.0 will be returned. In order for ChemApp to recognize H+ and H2 in the            |
    |        |                             | data-file, the names of the system components hydrogen and aqueous electron must be 'H' and 'EA', |
    |        |                             | respectively (see TQCNSC).                                                                        |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | CP,    | Heat capacity,              | Not valid for system components. Extensive properties for the equilibrium state dependent on the  |
    | H,     | Enthalpy,                   | inherent reference state of the data. However, when the entire system is selected and streams are |
    | S,     | Entropy,                    | used, the values represent balances, i.e. the extensive property for the equilibrium state minus  |
    | G,     | Gibbs energy,               | the sum of values for all streams. To obtain the total volume in this case, use option 'VT'.      |
    | V,     | Volume                      |                                                                                                   |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | CPM,   | Heat capacity/ amount unit, | Not valid for system components. Partial values are returned for constituents, integral values    |
    | HM,    | Enthalpy/ amount unit,      | for phases. Values are valid for default or selected amount units.                                |
    | SM,    | Entropy/ amount unit,       |                                                                                                   |
    | GM,    | Gibbs energy/ amount unit,  |                                                                                                   |
    | VM     | Volume/ amount unit         |                                                                                                   |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | X      | Fraction                    | Mass or mole fraction of a system component in the system, depending on the default or selected   |
    |        |                             | amount unit.                                                                                      |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | XP     | Fraction                    | Mass or mole fraction of a system component in a phase, depending on the default or selected      |
    |        |                             | amount unit.                                                                                      |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+
    | AP     | Equilibrium amount          | Equilibrium amount of (a) system component(s) in a phase.                                         |
    +--------+-----------------------------+---------------------------------------------------------------------------------------------------+


    """

    def __init__(self):
        """This class is not meant to be initialized."""

class SolutionModel(enum.Enum):
    """
    Available solution models.

    .. tabularcolumns:: |p{2cm}|p{7cm}|

    +-------+------------------------------------------------------------------------------------------------+
    | Model | Description                                                                                    |
    +=======+================================================================================================+
    | BDEF  | Binary defect formalism                                                                        |
    +-------+------------------------------------------------------------------------------------------------+
    | GAYE  | Gaye-Kapoor-Frohberg cell formalism with or without non-oxidic solutes                         |
    +-------+------------------------------------------------------------------------------------------------+
    | HOCH  | Hoch-Arpshofen formalism                                                                       |
    +-------+------------------------------------------------------------------------------------------------+
    | HOCHM | Hoch-Arpshofen formalism with magnetism                                                        |
    +-------+------------------------------------------------------------------------------------------------+
    | IDBS  | Gaseous model - C-H-O-S-N-Ar superfluid formalism                                              |
    +-------+------------------------------------------------------------------------------------------------+
    | IDMX  | Ideal mixing                                                                                   |
    +-------+------------------------------------------------------------------------------------------------+
    | IDVT  | Gaseous model - Virial equation with Tsonopoulos second virial coefficient correlation         |
    +-------+------------------------------------------------------------------------------------------------+
    | PURE  | Stoichiometric condensed phase                                                                 |
    +-------+------------------------------------------------------------------------------------------------+
    | QKTO  | General polynomial Kohler/Toop formalism                                                       |
    +-------+------------------------------------------------------------------------------------------------+
    | QKTOM | General polynomial Kohler/Toop formalism with magnetism                                        |
    +-------+------------------------------------------------------------------------------------------------+
    | QSOL  | Modified quasichemical formalism with nonoxidic solutes                                        |
    +-------+------------------------------------------------------------------------------------------------+
    | QUAS  | Modified quasichemical formalism                                                               |
    +-------+------------------------------------------------------------------------------------------------+
    | QUSL  | Two-sublattice equivalent fraction formalism as a polynomial                                   |
    +-------+------------------------------------------------------------------------------------------------+
    | QUSLM | Two-sublattice equivalent fraction formalism as a polynomial with magnetism                    |
    +-------+------------------------------------------------------------------------------------------------+
    | RKMP  | Redlich-Kister-Muggianu polynomial                                                             |
    +-------+------------------------------------------------------------------------------------------------+
    | RKMPM | Redlich-Kister-Muggianu polynomial with magnetism                                              |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBE  | Extended compound energy formalism                                                             |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBEM | Extended compound energy formalism with magnetism                                              |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBG  | Quadruplet quasichemical model                                                                 |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBGM | Quadruplet quasichemical model with magnetism                                                  |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBI  | Two-sublattice ionic formalism                                                                 |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBL  | Compound energy formalism                                                                      |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBLM | Compound energy formalism with magnetism                                                       |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBM  | Two-sublattice equivalent fraction formalism                                                   |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBMM | Two-sublattice equivalent fraction formalism with magnetism                                    |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBO  | Two-sublattice order/disorder formalism                                                        |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBOM | Two-sublattice order/disorder formalism with magnetism                                         |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBQ  | Quadruplet quasichemical model with composition-dependent coordination numbers                 |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBQM | Quadruplet quasichemical model with composition-dependent coordination numbers with magnetism  |
    +-------+------------------------------------------------------------------------------------------------+
    | SUBS  | Species chemical potential/bond energy formalism                                               |
    +-------+------------------------------------------------------------------------------------------------+
    | WAGN  | Unified interaction parameter formalism                                                        |
    +-------+------------------------------------------------------------------------------------------------+
    | WAGNM | Unified interaction parameter formalism with magnetism                                         |
    +-------+------------------------------------------------------------------------------------------------+
    | HELZ  | Aqueous model - Revised Helgeson-Kirkham-Flowers formalism                                     |
    +-------+------------------------------------------------------------------------------------------------+
    | HTDZ  | Aqueous model - Helgeson-Tanger-Shock formalism (Davies)                                       |
    +-------+------------------------------------------------------------------------------------------------+
    | HTSZ  | Aqueous model - Helgeson-Tanger-Shock formalism (ideal)                                        |
    +-------+------------------------------------------------------------------------------------------------+
    | HTWZ  | Aqueous model - Helgeson-Tanger-Shock formalism (Debye-Hueckel)                                |
    +-------+------------------------------------------------------------------------------------------------+
    | IDDZ  | Aqueous model - Davies formalism                                                               |
    +-------+------------------------------------------------------------------------------------------------+
    | IDWZ  | Aqueous model - Ideal aqueous mixing                                                           |
    +-------+------------------------------------------------------------------------------------------------+
    | PIHZ  | Aqueous model - Helgeson-Tanger-Shock formalism (Pitzer)                                       |
    +-------+------------------------------------------------------------------------------------------------+
    | PIMZ  | Aqueous model - Pitzer formalism                                                               |
    +-------+------------------------------------------------------------------------------------------------+
    | PITZ  | Aqueous model - Pitzer formalism with fixed alpha-1 and alpha-2                                |
    +-------+------------------------------------------------------------------------------------------------+
    | PIWZ  | Aqueous model - Pitzer formalism without E-theta and E-theta'                                  |
    +-------+------------------------------------------------------------------------------------------------+
    | SITZ  | Aqueous model - Specific ion-interaction formalism                                             |
    +-------+------------------------------------------------------------------------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class Status(enum.Enum):
    """
    Status values for phases and phase constituents.

    .. tabularcolumns:: |p{3cm}|p{8cm}|

    +------------+--------------------------------------------------------------------------+
    | Status     | Description                                                              |
    +============+==========================================================================+
    | ENTERED    | The phase or constituent is included in the equilibrium calculation.     |
    |            | An included phase is not necessarily stable at equilibrium.              |
    +------------+--------------------------------------------------------------------------+
    | DORMANT    | The phase or constituent is excluded from the mass balances (it will     |
    |            | appear with zero amount at equilibrium), but its activity is calculated. |
    |            | If the calculated activity for a phase is greater than one, it would be  |
    |            | stable if entered.                                                       |
    +------------+--------------------------------------------------------------------------+
    | ELIMINATED | The phase or constituent is ignored in the equilibrium calculation       |
    +------------+--------------------------------------------------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class StreamState(ConvertableObject):
    """
    This class represents the state of a stream used as input to a stream
    equilibrium calculation (StreamCalculation).

    Parameters
    ----------
    name : str
        stream name
    T : float
        stream temperature
    P : float
        stream pressure

    """

    def combine_with(self, other: StreamState, name: str = "") -> StreamState:
        """
        Add another stream's phase constituent to this stream and return the
        result.

        The StreamState object for which this function is called remains unchanged.

        The returned Cp, H, S, G, V property is set to 'nan'.

        Parameters
        ----------
        other
            The other StreamState object to combine.
        name
            A name for the returned new StreamState object. If empty, keep 'self' name.


        Returns
        -------
        StreamState
            copy of this stream including the incoming phase amounts from 'other'

        """

    def remove_above_threshold(
        self, threshold: float = 0.001, inplace: bool = False
    ) -> None | StreamState:
        """
        Remove all phase constituents with amounts above the given threshold.

        Parameters
        ----------
        threshold
            the threshold for removal (default=0.001)
        inplace
            whether a copy should be returned or the object itself is modified
            (default=False)

        Returns
        -------
        StreamState
            object with the phase constituents below the given threshold


        """

    def remove_below_threshold(
        self, threshold: float = 0.001, inplace: bool = False
    ) -> None | StreamState:
        """
        Remove all phase constituents with amounts below the given threshold.

        Parameters
        ----------
        threshold
            the threshold for removal (default=0.001)
        inplace
            whether a copy should be returned or the object itself is modified
            (default=False)

        Returns
        -------
        StreamState
            object with the phase constituents above the given threshold.

        """

    def split_at_threshold(
        self, threshold: float = 0.001
    ) -> Tuple[StreamState, StreamState]:
        """
        Split a stream into two streams.

        The two streams will be  named <name>_lower and <name>_higher, with respect to
        phase constituent amounts and given threshold.

        Parameters
        ----------
        threshold
            value at which to split (default=0.001)


        Returns
        -------
        Tuple[StreamState, StreamState]
            tuple of StreamState. All phase constituents with amounts below
            `threshold` are in the first item.


        """

    def split_by_states(self) -> Tuple[StreamState, StreamState, StreamState]:
        """
        Split a stream into three streams named <name>_liquid, <name>_solid, <name>_gas by
        their respective state of matter.

        Returns
        -------
        Tuple[StreamState, StreamState, StreamState]
            (solids, liquids, gaseous) tuple of StreamState with the respective
            matter of state.


        """

    @property
    def A(self) -> float:
        """
        Total stream amount.

        Returns
        -------
        float
            stream amount
        """

    @property
    def A_pcs(self) -> List[PhaseConstituentIncomingAmountState]:
        """
        Phase constituent amounts.

        Returns
        -------
        List[PhaseConstituentIncomingAmountState]
            phase constituent amounts


        """

    @property
    def name(self) -> str:
        """
        Stream name.

        Returns
        -------
        str
            name

        """

    @property
    def units(
        self,
    ) -> Dict[
        str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit
    ]:
        """
        The units used for the stream.

        Returns
        -------
        Dict[str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit]
            units used
        """

class StreamVariable(enum.Enum):
    """
    Available variables for stream properties that can be extracted.

    .. tabularcolumns:: |p{2cm}|p{3cm}|p{8cm}|

    +--------+---------------+-------------------------+
    + Option | Variable      | Unit                    |
    +========+===============+=========================+
    + CP     | Heat capacity | [current energy unit]/K |
    +--------+---------------+-------------------------+
    + H      | Enthalpy      | [current energy unit]   |
    +--------+---------------+-------------------------+
    + S      | Entropy       | [current energy unit]/K |
    +--------+---------------+-------------------------+
    + G      | Gibbs energy  | [current energy unit]   |
    +--------+---------------+-------------------------+
    + V      | Volume        | [current volume unit]   |
    +--------+---------------+-------------------------+

    """

class SystemComponentConfig(ObjectDescriptor):
    """
    This class represents the configuration of a system component as read from a
    file.

    Parameters
    ----------
    index : int
        system component one-based index
    name : str
        system component name
    mm : float
        system component molar mass

    """

    @property
    def mm(self) -> float:
        """
        The system component's molar mass.

        Returns
        -------
        float
            molar mass
        """

class SystemComponentIncomingAmountState(IncomingAmountState):
    """
    This class represents the state of a system component incoming amount in
    an equilibrium calculation result.

    Parameters
    ----------
    sc : str
        system component name
    A : float
        amount value

    """

    @property
    def sc(self) -> str:
        """
        System component name.

        Returns
        -------
        str
            name
        """

class SystemComponentState(ObjectDescriptor):
    """
    This class represents the state of a system component in an equilibrium
    calculation result.

    Parameters
    ----------
    index : int
        phase one-based index
    name : str
        phase name

    """

class TargetVariable(enum.Enum):
    """
    Variables that can be used in target calculations to arrive at a specified
    target condition.

    .. tabularcolumns:: |p{1cm}|p{3cm}|p{8cm}|

    +----------+--------------------+-------------------------------------------------------------+
    | Option   | Variable           | Description                                                 |
    +==========+====================+=============================================================+
    | P        | Total pressure     | Pressure is a target variable. The value VALS(1) serves as  |
    |          |                    | an initial estimate, VALS(2) is not used.                   |
    +----------+--------------------+-------------------------------------------------------------+
    | V        | Total volume       | Volume is a target variable. The value VALS(1) serves as an |
    |          |                    |  initial estimate, VALS(2) is not used.                     |
    +----------+--------------------+-------------------------------------------------------------+
    | T        | Temperature        | Temperature is a target variable. The value VALS(1) serves  |
    |          |                    | as an initial estimate, VALS(2) is not used.                |
    +----------+--------------------+-------------------------------------------------------------+
    | IA, IA0, | Incoming amount,   | Incoming amount is a target variable. The values VALS(1)    |
    | MU       | Chemical potential | and VALS(2) denote lower and upper limits, respectively.    |
    |          |                    | When defining streams, the constituent is included in the   |
    |          |                    | last stream considered. If incoming amounts for more then   |
    |          |                    | one substance need to be defined, all except the last one   |
    |          |                    | have to be called using 'IA0' as option to TQCE/TQCEL. Once |
    |          |                    | 'IA' is passed for the last incoming amount, the first      |
    |          |                    | calculation is made.                                        |
    +----------+--------------------+-------------------------------------------------------------+
    | Blank    |                    | Calculate without any target variable, VALS is not used.    |
    +----------+--------------------+-------------------------------------------------------------+

    """

    def __init__(self):
        """This class is not meant to be initialized."""

class ThermochemicalSystemConfig(ConvertableObject):
    """
    An object of this class stores all the information about a thermochemical
    system as it is read from a thermochemical data file.

    Parameters
    ----------
    file_path : str
        the path of the thermochemical data file
    file_expiry_date : float
        the thermochemical data file expiry date
    units : Dict[str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit]
        the units on which this object is based

    """

    @property
    def file_content(self) -> str:
        """
        The binary content of the thermochemical data file from which the
        thermochemical system was loaded.

        """

    @property
    def file_expiry_date(self) -> float:
        """
        The expiry date of the thermochemical data file from which the
        thermochemical system was loaded.

        """

    @property
    def file_md5(self) -> str:
        """
        The MD5 hash of the thermochemical data file from which the
        thermochemical system was loaded.

        """

    @property
    def file_name(self) -> str:
        """
        The name of the thermochemical data file from which the thermochemical
        system was loaded.

        """

    @property
    def file_path(self) -> str:
        """
        The path of the thermochemical data file from which the thermochemical
        system was loaded.

        """

    @property
    def phs(self) -> Dict[str, PhaseConfig]:
        """
        Dictionary of phases.
        """

    @property
    def scs(self) -> Dict[str, SystemComponentConfig]:
        """
        Dictionary of system components.
        """

    @property
    def units(
        self,
    ) -> Dict[
        str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit
    ]:
        """
        The ChemApp units that were active when this object was created.

        All numbers stored in this object and its child objects are in these
        units.

        """

def find_state(phase_name: str) -> PhaseMatterState:
    """
    Get the physical state of a phase, guessed by the phase name.

    Typically, e.g. liquid phase descriptions of 'compounds' are indicated by
    the phase name ending in _l or _(l). Similarly, gaseous species are very
    likely to be named _g or _(g). This function uses these and more assumptions
    to guess the state of a phase.

    Notice that matter of state is not a set property of the thermodynamic
    datafiles, and therefore only (educated) guesses are possible. If unsure, a
    phase name is assumed to be condensed/solid.

    Returns
    -------
    PhaseMatterState
        The likely state of the phase.
    """

def switch_components(
    composition: str, components: List[str], tol: float = 1e-5, normalize: bool = False
) -> List[float]:
    """
    Determine the stoichiometric factors for a composition given a set of
    arbitrary components.

    This function returns the numeric factors needed to create `composition`
    from the list of chemical formulae given in `components`.

    Parameters
    ----------
    composition
        chemical formulae of the composition to be decomposed
    components
        components which are linearly combined to form `composition`
    tol
        tolerance value for matching exact stoichiometries (default=1e-5)
    normalize
        scale the sum of stoichiometric factors to 1 (default=false)

    Examples
    --------
    >>> switch_components("Fe3O4", ["FeO", "O2"])
    [3.0, 2.0]

    >>> switch_components("SrNaCl3", ["NaCl", "SrCl2"])
    [1.0, 1.0]

    >>> switch_components("SrNaCl3", ["NaCl", "Sr", "Cl"])
    [1.0, 1.0, 2.0]

    Notice that the method uses a least square method internally to find a
    solution. Therefore, it is not necessary for the components to be orthogonal
    to each other, e.g. one component can be a linear combination of others.

    >>> switch_components("SrNaCl3", ["NaCl", "Cl2","Sr", "Cl"])
    [1.0, 0.8, 1.0, 0.4]

    However, in that case the solution is obviously not unique. Using the
    "normalize" option will not change that.

    >>> switch_components("Fe3O4", ["FeO", "O2"], normalize=True)
    [0.8571.., 0.1428..]

    Using the `tol` option allows to approximate factors, e.g. in case of
    non-integer component compositions:

    >>> switch_components("Al2O5", ["Al1.125O", "O"], tol=1e-2)
    [1.78, 3.22]

    Returns
    -------
    List[float]
        The stoichiometric factors for `components` that will form `composition`
    Raises
    ------
    ValueError
        In case there does not exist a rational positive solution given the inputs.
    """

class Object:
    """
    This class provides base infrastructure for all other classes in this
    module.
    """

class ObjectDescriptor(Object):
    """
    This class provides base features to describe an object in a thermochemical
    system, e.g. a system component, phase, or phase constituent.

    Parameters
    ----------
    index : int
        zero-based index
    name : str
        name

    """

    @property
    def index(self) -> int:
        """
        Zero-based index.

        Returns
        -------
        int
            index
        """

    @property
    def name(self) -> str:
        """
        Name.

        In the case of a chemical compound or ion, the chemical formula is used.

        Returns
        -------
        str
            name
        """

class ConvertableObject(Object):
    @classmethod
    def create_from(cls, src, format: DataFormat = ..., **kwargs) -> None:
        """
        Create an object from the specified source.

        Parameters
        ----------
        format
            the data format to use
        src
            the source object
        kwargs
            dictionary of optional keyword arguments

        Returns
        -------
        ConvertableObject
            instance of this class

        """

    def convert_to(self, format: DataFormat = ...) -> ConvertableObject:
        """
        Convert this object to an object of the specified format.

        Parameters
        ----------
        format
            the data format to use (DataFormat enumeration)

        Returns
        -------
        ConvertableObject
            an object in the specified format

        """
