"""

This module provides a friendly Python interface to the ChemApp library.

The friendly module uses static classes and descriptive names to to make the
interface clear, easy to read, and easy to remember.


Static Classes
--------------
These classes are simply containers that group related methods together to make
it easier to think about, remember, and use the interface. It is not possible
to instantiate these classes, i.e. you cannot create objects from them.

The following static classes are provided in the friendly interface:

* Info

  You can use this class to get information about ChemApp itself, its license,
  user name, expiry date, array sizes, etc.

* Units

  You can use this class to get and set the units used by ChemApp.

* ThermochemicalSystem

  With this class you can get information about the current thermochemical
  system that was read from a data file.


* EquilibriumCalculation

  You can use this class to do standard equilibrium calculations. 'Standard'
  means that the calculation does not use streams, and it also does not do
  phase map calculations. These calculations are absolute, which means that
  you can calculate Cp, H, S, G, and V for a system, but no not
  :math:`\Delta\text{Cp}`, :math:`\Delta\text{H}`, :math:`\Delta\text{S}`,
  :math:`\Delta\text{G}`, or :math:`\Delta\text{V}`.

* StreamCalculation

  You can use this class to do equilibrium calculations that take streams as
  inputs. You can combine ore, reductant, and flux streams, for example. With
  these calculations you can calculate both absolute and relative values of
  thermodynamic properties (Cp, H, S, G, V; and :math:`\Delta\text{Cp}`,
  :math:`\Delta\text{H}`, :math:`\Delta\text{S}`, :math:`\Delta\text{G}`,
  :math:`\Delta\text{V}`).

* PhaseMapCalculation

  This class assist with doing one-dimensional phase map calculations. These
  calculations are useful for finding phase boundaries.


Additionally, these classes exist for technical reasons and internal use.

* EquilibriumCalculationBase (abstract)

  This is an abstract class, and you should not use it directly. It is a base
  class of the EquilibriumCalculation, StreamCalculation, and
  PhaseMapCalculation classes, and provides methods used by all of them.

* AbsoluteEquilibriumCalculationBase (abstract)

  This is an abstract class, and you should not use it directly. It is a base
  class of the EquilibriumCalculation, and PhaseMapCalculation classes, and
  provides methods used by them.


Descriptive Naming
------------------
The friendly module uses longer names for parameters and functions than ChemApp
itself. The aim is to make these names easier to read, remember and use.

The following abbreviations are used in the interface to name functions,
parameters, etc.:

+--------------+--------------------+--------------+---------------------+
| Abbreviation | Term               | Abbreviation | Quantity            |
+==============+====================+==============+=====================+
| cf           | complex formula    | CP           | heat capacity       |
+--------------+--------------------+--------------+---------------------+
| cfs          | complex formulae   | E            | energy              |
+--------------+--------------------+--------------+---------------------+
| eq           | equilibrium        | G            | Gibbs energy        |
+--------------+--------------------+--------------+---------------------+
| mm           | molar mass         | H            | enthalpy            |
+--------------+--------------------+--------------+---------------------+
| ph           | phase              | IA           | incoming amount     |
+--------------+--------------------+--------------+---------------------+
| phs          | phases             | MU           | chemical potential  |
+--------------+--------------------+--------------+---------------------+
| pc           | phase constituent  | P            | pressure            |
+--------------+--------------------+--------------+---------------------+
| pcs          | phase constituents | S            | entropy             |
+--------------+--------------------+--------------+---------------------+
| sc           | system component   | T            | temperature         |
+--------------+--------------------+--------------+---------------------+
| scs          | system components  | V            | volume              |
+--------------+--------------------+--------------+---------------------+
| st           | stream             | VT           | total volume        |
+--------------+--------------------+--------------+---------------------+
| sts          | streams            | X            | fraction            |
+--------------+--------------------+--------------+---------------------+
| tg           | target             | XT           | mole fraction       |
+--------------+--------------------+--------------+---------------------+
| tx           | transition         | y            | mass fraction       |
+--------------+--------------------+--------------+---------------------+
| txs          | transitions        |              |                     |
+--------------+--------------------+--------------+---------------------+
"""

from __future__ import annotations
from os import PathLike
from typing import Dict, List, Tuple

import builtins as __builtins__
import chemapp.core
from chemapp.core import AmountUnit
from chemapp.core import ChemAppError
from chemapp.core import ChemAppInfo
from chemapp.core import ChemAppPyError
from chemapp.core import ChemicalFormulaIncomingAmountState
from chemapp.core import Composition
from chemapp.core import ConditionVariable
from chemapp.core import ConfigurationOption
from chemapp.core import EnergyUnit
from chemapp.core import EquilibriumCalculationResult
from chemapp.core import InteractionVariable
from chemapp.core import IoOption
from chemapp.core import PhaseConfig
from chemapp.core import PhaseConstituentConfig
from chemapp.core import PhaseConstituentData
from chemapp.core import PhaseConstituentIncomingAmountState
from chemapp.core import PhaseConstituentState
from chemapp.core import PhaseConstituentVariable
from chemapp.core import PhaseMapVariable
from chemapp.core import PhaseState
from chemapp.core import PressureUnit
from chemapp.core import Quantity
from chemapp.core import ResultVariable
from chemapp.core import SolutionModel
from chemapp.core import Status
from chemapp.core import StreamCalculationResult
from chemapp.core import StreamState
from chemapp.core import StreamVariable
from chemapp.core import SystemComponentConfig
from chemapp.core import SystemComponentIncomingAmountState
from chemapp.core import SystemComponentState
from chemapp.core import TargetVariable
from chemapp.core import TemperatureUnit
from chemapp.core import ThermochemicalSystemConfig
from chemapp.core import VolumeUnit
from chemapp.core import switch_components

import typing

__all__ = [
    "AbsoluteEquilibriumCalculationBase",
    "AmountUnit",
    "ChemAppError",
    "ChemAppInfo",
    "ChemAppPyError",
    "ChemicalFormulaIncomingAmountState",
    "Composition",
    "ConditionVariable",
    "ConfigurationOption",
    "EnergyUnit",
    "EquilibriumCalculation",
    "EquilibriumCalculationBase",
    "EquilibriumCalculationResult",
    "Info",
    "InteractionVariable",
    "IoOption",
    "PhaseConfig",
    "PhaseConstituentConfig",
    "PhaseConstituentData",
    "PhaseConstituentIncomingAmountState",
    "PhaseConstituentState",
    "PhaseConstituentVariable",
    "PhaseMapCalculation",
    "PhaseMapVariable",
    "PhaseState",
    "PressureUnit",
    "Quantity",
    "ResultVariable",
    "SolutionModel",
    "Status",
    "StreamCalculation",
    "StreamCalculationResult",
    "StreamState",
    "StreamVariable",
    "SystemComponentConfig",
    "SystemComponentIncomingAmountState",
    "SystemComponentState",
    "TargetVariable",
    "TemperatureUnit",
    "ThermochemicalSystem",
    "ThermochemicalSystemConfig",
    "Units",
    "VolumeUnit",
    "switch_components",
]

class AbsoluteEquilibriumCalculationBase(EquilibriumCalculationBase):
    """

    This abstract class provides base functionality for absolute equilibrium
    calculations, such as EquilibriumCalculation and PhaseMapCalculation.

    StreamCalculation is classified as relative, since its streams provide
    initial conditions relative to which extensive variable changes can be
    calculated.

    """

    @classmethod
    def _get_result_IAs(
        cls,
    ) -> List[PhaseConstituentIncomingAmountState | SystemComponentIncomingAmountState]:
        """
        Extract all the non-zero incoming amounts into a list.

        Returns
        -------
        List[PhaseConstituentIncomingAmountState | SystemComponentIncomingAmountState]
            List of PhaseConstituentIncomingAmountState or
            SystemComponentIncomingAmountState
        """

    @classmethod
    def _set_eq_condition_pc(
        cls, variable: ConditionVariable, ph: str | int, pc: str | int, value: float
    ) -> int:
        """
        Set an equilibrium condition on a phase constituent.

        Parameters
        ----------
        variable
            The variable to set.
        ph
            Phase name or zero-based index.
        pc
            Phase constituent name or zero-based index.
        value
            The value to set the variable to.

        Returns
        -------
        int
            Equilibrium condition number.
        """

    @classmethod
    def _set_eq_condition_sc(
        cls, variable: ConditionVariable, sc: str | int, value: float
    ) -> int:
        """
        Set an equilibrium condition on a system component.

        Parameters
        ----------
        variable
            The variable to set.
        sc
            The system component name or zero-based index.
        value
            The value to set the variable to.

        Returns
        -------
            The equilibrium condition number.
        """

    @classmethod
    def get_result_object(
        cls, result_type=chemapp.core.EquilibriumCalculationResult, error: bool = False
    ) -> EquilibriumCalculationResult | StreamCalculationResult:
        """
        Extract an equilibrium calculation's results in the form of a
        chemapp.core.EquilibriumCalculationResult object.

        Returns
        -------
        EquilibriumCalculationResult | StreamCalculationResult
            An object that holds all numeric results of a calculation
        """

    @classmethod
    def set_IA_cfs(cls, cfs: List[str], values: List[float]) -> List[int]:
        """
        Set chemical formula incoming amounts.

        The chemical formulas input can be in formats such as NaOH3, Fe2O3,
        Cr0.3Mn0.7C, or Na(OH)2, which are all accepted as long as the composition
        can be expressed using the system components. Special care needs to be taken
        given that the resulting amounts of the system components will be depending on
        the units used and the correct input of molar masses.

        Parameters
        ----------
        cfs
            list of chemical formulae
        value
            incoming amount values

        Returns
        -------
        List[int]
            equilibrium condition numbers, one for each system component

        """

    @classmethod
    def set_IA_pc(cls, ph: str | int, pc: str | int, value: float) -> int:
        """
        Set the incoming amount of a phase constituent.

        Parameters
        ----------
        ph
            The name or zero-based index of the phase.
        pc
            The name or zero-based index of the phase constituent.
        value
            The incoming amount value.

        Returns
        -------
        int
            The equilibrium condition number.
        """

    @classmethod
    def set_IA_sc(cls, sc: str | int, value: float) -> int:
        """
        Set the system component's incoming amount.

        Parameters
        ----------
        sc
            system component name or zero-based index
        value
            incoming amount value

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def set_eq_AC_pc(cls, ph: str | int, pc: str | int, value: float) -> int:
        """
        Set the phase constituent's equilibrium activity.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            constituent name or zero-based index
        value
            activity value

        Returns
        -------
        int
            Equilibrium condition number

        """

    @classmethod
    def set_eq_AC_sc(cls, sc: str | int, value: float) -> int:
        """
        Set the system component's equilibrium activity.

        Parameters
        ----------
        sc
            system component name or zero-based index
        value
            equilibrium activity value

        Returns
        -------
        int
            Equilibrium condition number

        """

    @classmethod
    def set_eq_MU_pc(cls, ph: str | int, pc: str | int, value: float) -> int:
        """
        Set the phase constituent's equilibrium chemical potential.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index
        value
            chemical potential value

        Returns
        -------
        int
            Equilibrium condition number

        """

class EquilibriumCalculation(AbsoluteEquilibriumCalculationBase):
    """
    This class provides facilities to perform normal (non-stream, non-phase
    map) equilibrium calculations.

    """

    @classmethod
    def calculate_eq(
        cls,
        print_results: bool = False,
        estimate: bool = False,
        return_result: bool = False,
        **kwargs,
    ) -> None | EquilibriumCalculationResult:
        """Calculate the equilibrium.

        Any keyword arguments for get_result_object can be added and will be
        passed to the result object if `return_result` is true.

        Parameters
        ----------
        print_results
            Indicates whether the results should be printed to
            console or not.
        estimate
            Indicates whether to use the previous calculation's
            results as an initial estimate.
        return_result
            Indicates whether the routine immediately returns a
            ResultObject.
        **kwargs
            arbitrary keyword argument list, handed through to the
            result object creation.

        Returns
        -------
        EquilibriumCalculationResult
            if return_result is set to `True`
        """

    @classmethod
    def calculate_eq_IA(
        cls, IA_ranges: List[Tuple[str, str, float, float]], print_results=False
    ) -> List[Tuple[str, str, float]]:
        """
        Search for the target by adjusting the specified incoming amounts.

        Parameters
        ----------
        IA_ranges
            List of incoming amount ranges.
            Each tuple should be (ph:str, pc:str, IA_min:float, IA_max:float)
        print_results
            Indicates whether the results should be printed to
            console or not (default is False)

        Returns
        -------
        List[Tuple[str, str, float]]
            incoming amounts. tuple list [(ph, pc, IA), ...]

        """

    @classmethod
    def calculate_eq_P(cls, P_guess: float, print_results: bool = False) -> float:
        """
        Search for the target by adjusting pressure, and use the provided guess as a starting value.

        Parameters
        ----------
        P_guess
            pressure guess value to start search
        print_results
            indicates whether the results should be printed to
            console or not

        Returns
        -------
        float
            resulting pressure

        """

    @classmethod
    def calculate_eq_T(cls, T_guess: float, print_results: bool = False) -> float:
        """
        Search for the target by adjusting temperature, and use the provided guess as a starting value.

        Parameters
        ----------

        T_guess
            temperature guess value to start search
        print_results
            indicates whether the results should be printed
            to console or not

        Returns
        -------
        float
            resulting temperature

        """

    @classmethod
    def calculate_eq_V(cls, V_guess: float, print_results: bool = False) -> float:
        """
        Search for the target by adjusting volume, and use the provided guess as a starting value.

        Parameters
        ----------
        V_guess
            volume guess value to start search
        print_results
            indicates whether the results should be printed
            to console or not

        Returns
        -------
        float
            resulting volume

        """

    @classmethod
    def set_target_AC_pc(cls, ph: str | int, pc: str | int, value: float):
        """
        Set the target as the activity of the specified phase constituent.

        Parameters
        ----------
        ph
            phase name or index
        pc
            phase constituent name or index
        value
            activity value

        """

    @classmethod
    def set_target_XT_pc(cls, ph: str | int, pc: str | int, value: float):
        """
        Set the target as the mole fraction of the specified phase constituent.

        Parameters
        ----------
        ph
            phase name or index
        pc
            phase consituent name or index
        value
            mole fraction value

        """

    @classmethod
    def set_target_XT_sc_in_ph(cls, sc: str | int, ph: str | int, value: float):
        """
        Set the target as the mole fraction of the specified system component in the specified phase.

        Parameters
        ----------
        sc
            system component name or index
        ph
            phase name or index
        value
            mole fraction value

        """

    @classmethod
    def set_target_formation_of_ph(cls, ph: str | int):
        """
        Set the target as formation of the specified phase.

        Parameters
        ----------
        ph
            phase name or index
        """

    @classmethod
    def set_target_precipitation_from_ph(cls, ph: str | int):
        """
        Set the target as precipitation from the specified phase.

        Parameters
        ----------
        ph
            phase name or index

        """

    @classmethod
    def set_tg_AC_pc(cls, ph: str | int, pc: str | int, value: float):
        """
        Set the target as the activity of the specified phase constituent.

        FutureWarning: This function is renamed to `set_target_AC_pc`. `set_tg_AC_pc` will be removed in a future version.

        .. deprecated:: 826
            This function is renamed to `set_target_AC_pc`. `set_tg_AC_pc` will be removed in a future version.

        Parameters
        ----------
        ph : str | int
            phase name or index
        pc : str | int
            phase constituent name or index
        value : float
            activity value

        """

    @classmethod
    def set_tg_XT_pc(cls, ph: str | int, pc: str | int, value: float):
        """
        Set the target as the mole fraction of the specified phase constituent.

        FutureWarning: This function is renamed to `set_target_XT_pc`. `set_tg_XT_pc` will be removed in a future version.

        .. deprecated:: 826
            This function is renamed to `set_target_XT_pc`. `set_tg_XT_pc` will be removed in a future version.


        Parameters
        ----------
        ph
            phase name or index
        pc
            phase consituent name or index
        value
            mole fraction value

        """

    @classmethod
    def set_tg_XT_sc_in_ph(cls, sc: str | int, ph: str | int, value: float):
        """
        Set the target as the mole fraction of the specified system component in
        the specified phase.

        FutureWarning: This function is renamed to `set_target_XT_sc_in_ph`. `set_tg_XT_sc_in_ph` will be removed in a future version.

        .. deprecated:: 826
            This function is renamed to `set_target_XT_sc_in_ph`. `set_tg_XT_sc_in_ph` will be removed in a future version.

        Parameters
        ----------
        sc
            system component name or index
        ph
            phase name or index
        value
            mole fraction value

        """

    @classmethod
    def set_tg_formation_of_ph(cls, ph: str | int):
        """
        Set the target as formation of the specified phase.

        FutureWarning: This function is renamed to `set_target_formation_of_ph`. `set_tg_formation_of_ph` will be removed in a future version.

        .. deprecated:: 826
            This function is renamed to `set_target_formation_of_ph`. `set_tg_formation_of_ph` will be removed in a future version.

        Parameters
        ----------
        ph
            phase name or index

        """

    @classmethod
    def set_tg_precipitation_from_ph(cls, ph: str | int):
        """
        Set the target as precipitation from the specified phase.

        FutureWarning: This function is renamed to `set_target_precipitation_from_ph`. `set_tg_precipitation_from_ph` will be removed in a future version.

        .. deprecated:: 826
            This function is renamed to `set_target_precipitation_from_ph`. `set_tg_precipitation_from_ph` will be removed in a future version.

        Parameters
        ----------
        ph
            phase name or index

        """

class EquilibriumCalculationBase:
    """
    This class provides base functionality share between all equilibrium
    calculation classes.

    """

    dt = None
    t_end = None
    t_start = None
    result_type = chemapp.core.EquilibriumCalculationResult
    @classmethod
    def _end_calc(cls):
        """
        End the calculation, initiate results fetching and timing.

        """

    @classmethod
    def _get_result_ph_pcs(
        cls, i_ph: int, A_ph: float
    ) -> Dict[str, PhaseConstituentState]:
        """
        Extract all the phase constituent states for the specified phase into
        a dictionary.

        Parameters
        ----------
        i_ph
            one-based phase index
        A_ph
            phase amount to scale result

        Returns
        -------
        Dict[str, PhaseConstituentState]
            Phase constituent states for all phases
        """

    @classmethod
    def _get_result_ph_scs(
        cls, i_ph: int, A_ph: float
    ) -> Dict[str, SystemComponentState]:
        """
        Extract all the system component states for the specified phase into
        a dictionary.

        Parameters
        ----------
        i_ph
            one-based phase index
        A_ph
            phase amount to scale result

        Returns
        -------
        Dict[str, SystemComponentState]
            System component states for all system components of the phase

        """

    @classmethod
    def _get_result_phs(cls) -> Dict[str, PhaseState]:
        """
        Extract all the phase states into a dictionary.

        The PhaseState class constains all the information for phase
        constituents and system components for that phase.

        Returns
        -------
        Dict[str, PhaseState]
            A dictionary with phase names as keys, `PhaseState` objects as
            values.

        """

    @classmethod
    def _get_result_scs(cls) -> Dict[str, SystemComponentState]:
        """
        Extract all the system component states into a dictionary.

        The SystemComponentState object contains all result properties available
        for system components.

        Returns
        -------
        Dict[str, SystemComponentState]
            A dictionary with system component names as keys,
            `SystemComponentState` objects as values.

        """

    @classmethod
    def _set_eq_condition_ph(
        cls, variable: ConditionVariable, ph: str | int, value: float
    ) -> int:
        """
        Set an equilibrium condition on a phase.

        Parameters
        ----------
        variable
            the variable to set
        ph
            phase name or zero-based index
        value
            the value to set the variable to

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def _set_eq_condition_sys(cls, variable: ConditionVariable, value: float) -> int:
        """
        Set a system-level equilibrium condition.

        Parameters
        ----------
        variable
            the variable to set
        value
            the value to set the variable to

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def get_eq_A(cls) -> float:
        """
        Get the system's total amount at equilibrium.

        Returns
        -------
        float
            amount value

        """

    @classmethod
    def get_eq_AC_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's activity at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            activity value

        """

    @classmethod
    def get_eq_AC_pcs(cls) -> List[float]:
        """
        Get the activity at equilibrium for all phase constituents.

        All phase constituents of all phases as a continuous list.

        Returns
        -------
        List[float]
            activity values

        """

    @classmethod
    def get_eq_AC_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the activity at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            activity values

        """

    @classmethod
    def get_eq_AC_ph(cls, ph: str | int) -> float:
        """
        Get the phase's activity at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            activity value

        """

    @classmethod
    def get_eq_AC_phs(cls) -> List[float]:
        """
        Get the activity at equilibrium for all phases.

        Returns
        -------
        List[float]
            activity values

        """

    @classmethod
    def get_eq_AC_sc(cls, sc: str | int) -> float:
        """
        Get the system component's activity at equilibrium.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        float
            activity value

        """

    @classmethod
    def get_eq_AC_scs(cls) -> List[float]:
        """
        Get the activity at equilibrium for all system components.

        Returns
        -------
        List[float]
            activity values

        """

    @classmethod
    def get_eq_A_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            amount value

        """

    @classmethod
    def get_eq_A_pcs(cls) -> List[float]:
        """
        Get the amount at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            amount values

        """

    @classmethod
    def get_eq_A_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the amount at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            amount values

        """

    @classmethod
    def get_eq_A_ph(cls, ph: str | int) -> float:
        """
        Get the phase's amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            amount value

        """

    @classmethod
    def get_eq_A_phs(cls) -> List[float]:
        """
        Get the amount at equilibrium for all phases.

        Returns
        -------
        List[float]
            amount values

        """

    @classmethod
    def get_eq_A_sc(cls, sc: str | int) -> float:
        """
        Get the system component's amount at equilibrium.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        float
            amount value

        """

    @classmethod
    def get_eq_A_sc_in_ph(cls, ph: str | int, sc: str | int) -> float:
        """
        Get the equilibrium amount of the specified system component in the
        specified phase.

        Parameters
        ----------
        sc
            system component name or index
        ph
            phase name or index

        Returns
        -------
        float
            amount value

        """

    @classmethod
    def get_eq_A_scs(cls) -> List[float]:
        """
        Get the amount at equilibrium for all system components.

        Returns
        -------
        List[float]
            list of amount values

        """

    @classmethod
    def get_eq_A_scs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the equilibrium amount of all system components in the specified
        phase.

        Parameters
        ----------
        ph
            phase name or index

        Returns
        -------
        List[float]
            amount values

        """

    @classmethod
    def get_eq_A_scs_in_phs(cls) -> Dict[str, List[float]]:
        """
        Get the equilibrium amount of all system components in all phases.


        Returns
        -------
        Dict[str,List[float]]
            amount values

        """

    @classmethod
    def get_eq_Cp(cls) -> float:
        """
        Get the system's heat capacity at equilibrium.

        Returns
        -------
        float
            heat capacity value

        """

    @classmethod
    def get_eq_CpM(cls) -> float:
        """
        Get the system's heat capacity per amount at equilibrium.

        Returns
        -------
        float
            heat capacity per amount value

        """

    @classmethod
    def get_eq_CpM_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's heat capacity per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            heat capacity per amount value

        """

    @classmethod
    def get_eq_CpM_pcs(cls) -> List[float]:
        """
        Get the heat capacity per amount at equilibrium for all phase
        constituents.

        Returns
        -------
        List[float]
            heat capacity per amount values

        """

    @classmethod
    def get_eq_CpM_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the heat capacity per amount at equilibrium for all phase
        constituents in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            heat capacity per amount values

        """

    @classmethod
    def get_eq_CpM_ph(cls, ph: str | int) -> float:
        """
        Get the phase's heat capacity per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            heat capacity per amount value

        """

    @classmethod
    def get_eq_CpM_phs(cls) -> List[float]:
        """
        Get the heat capacity per amount at equilibrium for all phases.

        Returns
        -------
        List[float]
            heat capacity per amount values

        """

    @classmethod
    def get_eq_Cp_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's heat capacity at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            heat capacity value

        """

    @classmethod
    def get_eq_Cp_pcs(cls) -> List[float]:
        """
        Get the heat capacity at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            heat capacity values

        """

    @classmethod
    def get_eq_Cp_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the heat capacity at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            heat capacity values

        """

    @classmethod
    def get_eq_Cp_ph(cls, ph: str | int) -> float:
        """
        Get the phase's heat capacity at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            heat capacity value

        """

    @classmethod
    def get_eq_Cp_phs(cls) -> List[float]:
        """
        Get the heat capacity at equilibrium for all phases.

        Returns
        -------
        List[float]
            heat capacity values

        """

    @classmethod
    def get_eq_G(cls) -> float:
        """
        Get the system's Gibbs energy at equilibrium.

        Returns
        -------
        float
            Gibbs energy value

        """

    @classmethod
    def get_eq_GM(cls) -> float:
        """
        Get the system's Gibbs energy per amount at equilibrium.

        Returns
        -------
        float
            Gibbs energy per amount value

        """

    @classmethod
    def get_eq_GM_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's Gibbs energy per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            Gibbs energy per amount value

        """

    @classmethod
    def get_eq_GM_pcs(cls) -> List[float]:
        """
        Get the Gibbs energy per amount at equilibrium for all phase
        constituents.

        Returns
        -------
        List[float]
            Gibbs energy per amount value

        """

    @classmethod
    def get_eq_GM_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the Gibbs energy per amount at equilibrium for all phase
        constituents in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            Gibbs energy per amount value

        """

    @classmethod
    def get_eq_GM_ph(cls, ph: str | int) -> float:
        """
        Get the phase's Gibbs energy per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            Gibbs energy per amount value

        """

    @classmethod
    def get_eq_GM_phs(cls) -> List[float]:
        """
        Get the Gibbs energy per amount at equilibrium for all phases.

        Returns
        -------
        List[float]
            Gibbs energy per amount value

        """

    @classmethod
    def get_eq_G_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's Gibbs energy at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            Gibbs energy value

        """

    @classmethod
    def get_eq_G_pcs(cls) -> float:
        """
        Get the Gibbs energy at equilibrium for all phase constituents.

        Returns
        -------
        float
            Gibbs energy value

        """

    @classmethod
    def get_eq_G_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the Gibbs energy at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            Gibbs energy value

        """

    @classmethod
    def get_eq_G_ph(cls, ph: str | int) -> float:
        """
        Get the phase's Gibbs energy at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            Gibbs energy value

        """

    @classmethod
    def get_eq_G_phs(cls) -> float:
        """
        Get the Gibbs energy at equilibrium for all phases.

        Returns
        -------
        float
            Gibbs energy value

        """

    @classmethod
    def get_eq_H(cls) -> float:
        """
        Get the system's enthalpy at equilibrium.

        Returns
        -------
        float
            enthalpy value

        """

    @classmethod
    def get_eq_HM(cls) -> float:
        """
        Get the system's enthalpy per amount at equilibrium.

        Returns
        -------
        float
            enthalpy per amount value

        """

    @classmethod
    def get_eq_HM_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's enthalpy per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            enthalpy per amount value

        """

    @classmethod
    def get_eq_HM_pcs(cls) -> List[float]:
        """
        Get the enthalpy per amount at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            enthalpy per amount values

        """

    @classmethod
    def get_eq_HM_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the enthalpy per amount at equilibrium for all phase constituents in
        the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            enthalpy per amount values

        """

    @classmethod
    def get_eq_HM_ph(cls, ph: str | int) -> float:
        """
        Get the phase's enthalpy per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            enthalpy per amount value

        """

    @classmethod
    def get_eq_HM_phs(cls) -> List[float]:
        """
        Get the enthalpy per amount at equilibrium for all phases.

        Returns
        -------
        List[float]
            enthalpy per amount values

        """

    @classmethod
    def get_eq_H_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's enthalpy at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            enthalpy value

        """

    @classmethod
    def get_eq_H_pcs(cls) -> List[float]:
        """
        Get the enthalpy at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            enthalpy values

        """

    @classmethod
    def get_eq_H_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the enthalpy at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            enthalpy values

        """

    @classmethod
    def get_eq_H_ph(cls, ph: str | int) -> float:
        """
        Get the phase's enthalpy at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            enthalpy value

        """

    @classmethod
    def get_eq_H_phs(cls) -> List[float]:
        """
        Get the enthalpy at equilibrium for all phases.

        Returns
        -------
        List[float]
            enthalpy values

        """

    @classmethod
    def get_eq_IA_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's incoming amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            incoming amount value

        """

    @classmethod
    def get_eq_IA_pcs(cls) -> List[float]:
        """
        Get the incoming amount at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            incoming amount values

        """

    @classmethod
    def get_eq_IA_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the incoming amount at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            incoming amount values

        """

    @classmethod
    def get_eq_MU_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's chemical potential at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            chemical potential value

        """

    @classmethod
    def get_eq_MU_pcs(cls) -> List[float]:
        """
        Get the chemical potential at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            chemical potential values

        """

    @classmethod
    def get_eq_MU_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the chemical potential at equilibrium for all phase constituents in
        the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            chemical potential values

        """

    @classmethod
    def get_eq_MU_ph(cls, ph: str | int) -> float:
        """
        Get the phase's chemical potential at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            chemical potential value

        """

    @classmethod
    def get_eq_MU_phs(cls) -> List[float]:
        """
        Get the chemical potential at equilibrium for all phases.

        Returns
        -------
        List[float]
            chemical potential value

        """

    @classmethod
    def get_eq_MU_sc(cls, sc: str | int) -> float:
        """
        Get the system component's chemical potential at equilibrium.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        float
            chemical potential value

        """

    @classmethod
    def get_eq_MU_scs(cls) -> List[float]:
        """
        Get the chemical potential at equilibrium for all system components.

        Returns
        -------
        List[float]
            list of chemical potential values

        """

    @classmethod
    def get_eq_P(cls) -> float:
        """
        Get the system's pressure at equilibrium.

        Returns
        -------
        float
            pressure value

        """

    @classmethod
    def get_eq_S(cls) -> float:
        """
        Get the system's entropy at equilibrium.

        Returns
        -------
        float
            entropy value

        """

    @classmethod
    def get_eq_SM(cls) -> float:
        """
        Get the system's entropy per amount at equilibrium.

        Returns
        -------
        float
            entropy per amount value

        """

    @classmethod
    def get_eq_SM_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's entropy per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            entropy per amount value

        """

    @classmethod
    def get_eq_SM_pcs(cls) -> List[float]:
        """
        Get the entropy per amount at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            entropy per amount values

        """

    @classmethod
    def get_eq_SM_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the entropy per amount at equilibrium for all phase constituents in
        the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            entropy per amount values

        """

    @classmethod
    def get_eq_SM_ph(cls, ph: str | int) -> float:
        """
        Get the phase's entropy per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            entropy per amount value

        """

    @classmethod
    def get_eq_SM_phs(cls) -> List[float]:
        """
        Get the entropy per amount at equilibrium for all phases.

        Returns
        -------
        List[float]
            entropy per amount values

        """

    @classmethod
    def get_eq_S_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's entropy at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            entropy value

        """

    @classmethod
    def get_eq_S_pcs(cls) -> List[float]:
        """
        Get the entropy at equilibrium for all phase constituents.

        Returns
        -------
        float
            entropy value

        """

    @classmethod
    def get_eq_S_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the entropy at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            entropy values

        """

    @classmethod
    def get_eq_S_ph(cls, ph: str | int) -> float:
        """
        Get the phase's entropy at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            entropy value

        """

    @classmethod
    def get_eq_S_phs(cls) -> List[float]:
        """
        Get the entropy at equilibrium for all phases.

        Returns
        -------
        List[float]
            entropy value

        """

    @classmethod
    def get_eq_T(cls) -> float:
        """
        et the system's temperature at equilibrium.

        Returns
        -------
        float
            temperature value

        """

    @classmethod
    def get_eq_V(cls) -> float:
        """
        Get the system's volume at equilibrium.

        Returns
        -------
        float
            volume value

        """

    @classmethod
    def get_eq_VM(cls) -> float:
        """
        Get the system's volume per amount at equilibrium.

        Returns
        -------
        float
            volume per amount value

        """

    @classmethod
    def get_eq_VM_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's volume per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            volume per amount value

        """

    @classmethod
    def get_eq_VM_pcs(cls) -> List[float]:
        """
        Get the volume per amount at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            volume per amount values

        """

    @classmethod
    def get_eq_VM_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the volume per amount at equilibrium for all phase constituents in
        the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            volume per amount values

        """

    @classmethod
    def get_eq_VM_ph(cls, ph: str | int) -> float:
        """
        Get the phase's volume per amount at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            volume per amount value

        """

    @classmethod
    def get_eq_VM_phs(cls) -> List[float]:
        """
        Get the volume per amount at equilibrium for all phases.

        Returns
        -------
        List[float]
            volume per amount values

        """

    @classmethod
    def get_eq_VT(cls) -> float:
        """
        Get the system's total volume at equilibrium.

        Returns
        -------
        float
            total volume value

        """

    @classmethod
    def get_eq_V_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the phase constituent's volume at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            volume value

        """

    @classmethod
    def get_eq_V_pcs(cls) -> List[float]:
        """
        Get the volume at equilibrium for all phase constituents.

        Returns
        -------
        List[float]
            volume values

        """

    @classmethod
    def get_eq_V_pcs_in_ph(cls, ph: str | int) -> float:
        """
        Get the volume at equilibrium for all phase constituents in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            volume value

        """

    @classmethod
    def get_eq_V_ph(cls, ph: str | int) -> float:
        """
        Get the phase's volume at equilibrium.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        float
            volume value

        """

    @classmethod
    def get_eq_V_phs(cls) -> List[float]:
        """
        Get the volume at equilibrium for all phases.

        Returns
        -------
        List[float]
            volume value

        """

    @classmethod
    def get_eq_X_sc(cls, sc: str | int) -> float:
        """
        Get the system component's amount fraction at equilibrium.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        float
            amount fraction value

        """

    @classmethod
    def get_eq_X_sc_in_ph(cls, ph: str | int, sc: str | int) -> float:
        """
        Get the equilibrium amount fraction of the specified system component
        in the specified phase.

        Parameters
        ----------
        sc
            system component name or index
        ph
            phase name or index

        Returns
        -------
        float
            amount fraction value

        """

    @classmethod
    def get_eq_X_scs(cls) -> List[float]:
        """
        Get the amount fraction at equilibrium for all system components.

        Returns
        -------
        List[float]
            list of amount fraction values

        """

    @classmethod
    def get_eq_X_scs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the equilibrium amount fraction of all system components in the
        specified phase.

        Parameters
        ----------
        ph
            phase name or index

        Returns
        -------
        List[float]
            amount fraction values

        """

    @classmethod
    def get_eq_X_scs_in_phs(
        cls,
    ) -> Dict[str, List[float]]:
        """
        Get the equilibrium amount fraction of all system components in all phases.


        Returns
        -------
        Dict[str,List[float]]
            amount fraction values

        """

    @classmethod
    def get_result_object(
        cls, result_type=chemapp.core.EquilibriumCalculationResult, error=False
    ) -> EquilibriumCalculationResult | StreamCalculationResult:
        """
        Extract an equilibrium calculation's results in the form of a
        `EquilibriumCalculationResult` object.

        The created object contains all system wide results as well as for each
        phase and system component, statically. These are static and not
        affected by any changes to the ThermochemicalSystem or
        EquilibriumCalculation classes.

        Refer to the `EquilibriumCalculationResult` documentation for further
        information on how to use the results.

        Parameters
        ----------
        result_type
            Default: EquilibriumCalculationResult, can be
            StreamCalculationResult. Only used for simpler inheritance.
        error
            Default: `False`, implies that results can be obtained. If `True`

        Returns
        -------
        EquilibriumCalculationResult
            An object that contains the result of the last equilibrium calculation.
        """

    @classmethod
    def get_str_inputs(cls) -> str:
        """
        Get the current set of ChemApp inputs as a string. This is the friendly
        equivalent of ChemApp's :py:meth:`~chemapp.basic.tqshow` function.

        Returns
        -------
        str
            current ChemApp inputs (same as tqshow output)

        """

    @classmethod
    def remove_eq_condition(cls, number: int):
        """
        Remove an existing equilibrium condition.

        Typically, the equilibrium condition number is returned from the
        respective condition setting call. Some conditions can not be removed.
        Generally, every condition can be replaced with a new value without
        being removed previously.

        Parameters
        ----------
        number
            The condition number

        """

    @classmethod
    def remove_eq_conditions_IA(cls):
        """
        Remove all existing conditions related to incoming amounts. This
        includes incoming amounts (IA), chemical potentials (MU), and relative
        activities (AC). It does not remove system-level conditions or targets.

        This is the equivalent of a call to :py:meth:`~chemapp.basic.tqremc(0)`.

        """

    @classmethod
    def remove_eq_conditions_all(cls):
        """
        Remove all existing equilibrium conditions.

        This action removes all conditions, targets, and previously set
        configuration options.

        This is the equivalent of a call to :py:meth:`~chemapp.basic.tqremc(-2)`.

        """

    @classmethod
    def set_eq_A_ph(cls, ph: str | int, value: float) -> int:
        """
        Set the phase's equilibrium amount.

        Parameters
        ----------
        ph
            phase name or zero-based index
        value
            amount value

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def set_eq_Cp(cls, value: float) -> int:
        """
        Set the equilibrium heat capacity for the system.

        Parameters
        ----------
        value
            heat capacity

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def set_eq_Cp_ph(cls, ph: str | int, value: float) -> int:
        """
        Set the phase's equilibrium heat capacity.

        Parameters
        ----------
        ph
            phase name or zero-based index
        value
            heat capacity value

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def set_eq_G(cls, value: float) -> int:
        """
        Set the equilibrium Gibbs energy for the system.

        Parameters
        ----------
        value
            Gibbs energy

        Returns
        -------
        int
            equilibrium condition number


        """

    @classmethod
    def set_eq_G_ph(cls, ph: str | int, value: float) -> int:
        """
        Set the phase's equilibrium Gibbs energy.

        Parameters
        ----------
        ph
            phase name or zero-based index
        value
            Gibbs energy value

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def set_eq_H(cls, value: float) -> int:
        """
        Set the equilibrium enthalpy for the system.

        Parameters
        ----------
        value
            enthalpy

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def set_eq_H_ph(cls, ph: str | int, value: float) -> int:
        """
        Set the phase's equilibrium enthalpy.

        Parameters
        ----------
        ph
            phase name or zero-based index
        value
            enthalpy value

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_eq_P(cls, value: float) -> int:
        """
        Set the equilibrium pressure for the system.

        Parameters
        ----------
        value
            pressure

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_eq_S(cls, value: float) -> int:
        """
        Set the equilibrium entropy for the system.

        Parameters
        ----------
        value
            entropy

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_eq_S_ph(cls, ph: str | int, value: float) -> int:
        """
        Set the phase's equilibrium entropy.

        Parameters
        ----------
        ph
            phase name or zero-based index
        value
            entropy value

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_eq_T(cls, value: float) -> int:
        """
        Set the equilibrium temperature for the system.

        Parameters
        ----------
        value
            temperature

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_eq_V(cls, value: float) -> int:
        """
        Set the equilibrium volume for the system.

        Parameters
        ----------
        value
            volume

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_eq_VT(cls, value: float) -> int:
        """
        Set the equilibrium total volume for the system.

        Parameters
        ----------
        value
            total volume

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_eq_V_ph(cls, ph: str | int, value: float) -> int:
        """
        Set the phase's equilibrium volume.

        Parameters
        ----------
        ph
            phase name or zero-based index
        value
            volume value

        Returns
        -------
        int
            equilibrium condition number
        """

class Info:
    """
    This class provides access to ChemApp functions that are used to get
    information about ChemApp itself, the user that it is licensed to, the
    license, and the array sizes that ChemApp uses to store data in memory.

    The class is static, which means that you cannot create objects with it
    (e.g. ``info_object = Info()``). The ``friendly`` module uses static classes
    as a convenient way to group ChemApp functions that perform related tasks
    together to make them easier to find and use.

    """

    @classmethod
    def get_array_size_max_dict(cls) -> Dict[str, int]:
        """
        Get ChemApp's maximum internal array dimensions as a dictionary.

        The keys are 'na', 'nb', 'nc', 'nd', 'ne', 'nf', 'ng', 'nh', 'ni', 'nj'
        and 'nk'.

        * na: constituents
        * nb: system components
        * nc: mixture phases
        * nd: excess Gibbs energy coefficients for a mixture phase
        * ne: Excess magnetic coefficients for a mixture phase
        * nf: Sublattices of a mixture phase
        * ng: Constituents of a sublattice
        * nh: Oxide constituents of a phase described by the Gaye-Kapoor-Frohberg
              or modified quasichemical formalisms.
        * ni: Gibbs energy/heat capacity equations for a constituent
        * nk: Constituents with P,T-dependent molar volumes

        Returns
        -------
        Dict[str,int]
            The internal array sizes of ChemApp

        """

    @classmethod
    def get_array_size_max_list(cls) -> List[int]:
        """
        Get ChemApp's maximum internal array dimensions as a list.

        Refer to `get_array_size_max_dict()` for a description of each value.

        Returns
        -------
        List[int]
            The internal array sizes of ChemApp

        """

    @classmethod
    def get_array_size_max_str(cls) -> str:
        """
        Print ChemApp's maximum internal array dimensions to a string.

        Returns
        -------
        str
            A printable table of the array sizes of ChemApp

        """

    @classmethod
    def get_array_size_used_dict(cls) -> Dict[str, int]:
        """
        Get ChemApp's currently used internal array dimensions as a dictionary.

        The keys are 'na', 'nb', 'nc', 'nd', 'ne', 'nf', 'ng', 'nh', 'ni', 'nj'
        and 'nk'.

        * na: constituents
        * nb: system components
        * nc: mixture phases
        * nd: excess Gibbs energy coefficients for a mixture phase
        * ne: Excess magnetic coefficients for a mixture phase
        * nf: Sublattices of a mixture phase
        * ng: Constituents of a sublattice
        * nh: Oxide constituents of a phase described by the Gaye-Kapoor-Frohberg
              or modified quasichemical formalisms.
        * ni: Gibbs energy/heat capacity equations for a constituent
        * nk: Constituents with P,T-dependent molar volumes

        Returns
        -------
        Dict[str, int]
            The currently used array sizes

        """

    @classmethod
    def get_array_size_used_list(cls) -> List[int]:
        """
        Get ChemApp's currently used internal array dimensions as a list.

        Refer to `get_array_size_used_dict()` for a description of each value.

        Returns
        -------
        List[int]
            Currently used array sizes of ChemApp


        """

    @classmethod
    def get_array_size_used_str(cls) -> str:
        """
        Print ChemApp's currently used internal array dimensions to a string.

        Returns
        -------
        str
            A printable table of the array sizes of ChemApp

        """

    @classmethod
    def get_copyright_message(cls) -> str:
        """
        Get the ChemApp copyright message.

        Returns
        -------
        str
            The copyright message

        """

    @classmethod
    def get_dongle_id(cls) -> str:
        """
        Get the ChemApp dongle id.

        Returns
        -------
        str
            ChemApp dongle id

        """

    @classmethod
    def get_dongle_type(cls) -> str:
        """
        Get the ChemApp dongle type.

        Returns
        -------
        str
            dongle type

        """

    @classmethod
    def get_expiration_date(cls) -> Tuple[int, int]:
        """
        Get the ChemApp license expiration date.

        Returns
        -------
        Tuple[int, int]
            (Month, Year) expiration date tuple.

        """

    @classmethod
    def get_object(cls) -> ChemAppInfo:
        """
        Extract the information about ChemApp into a `ChemAppInfo` object.

        Returns
        -------
        ChemAppInfo
            Class object containing all ChemApp information

        """

    @classmethod
    def get_program_id(cls) -> str:
        """
        Get the ChemApp program ID.

        Returns
        -------
        str
            ChemApp program ID

        """

    @classmethod
    def get_user_id(cls) -> str:
        """
        Get the ChemApp user ID.

        Returns
        -------
        str
            User ID

        """

    @classmethod
    def get_user_name(cls) -> str:
        """
        Get the ChemApp user name.

        Returns
        -------
        str
            ChemApp user name, license holder.

        """

    @classmethod
    def get_version_number(cls) -> str:
        """
        Get the ChemApp version number.

        Returns
        -------
        str
            `ChemApp` version number string (Major.Minor.Build)

        """

    @classmethod
    def is_light(cls):
        """
        Indicates whether the Light version of ChemApp is being used.

        This is only used as a failsafe in documentation examples and has
        no application for users.

        Returns
        -------
        bool
            True if ChemApp light limitations apply

        """

class PhaseMapCalculation(AbsoluteEquilibriumCalculationBase):
    class IA_range(tuple):
        """
        IA_range(ph: str, pc, start, end)
        """

        __match_args__: typing.ClassVar[tuple] = ("ph", "pc", "start", "end")
        __slots__: typing.ClassVar[tuple] = tuple()
        _field_defaults: typing.ClassVar[dict] = {}
        _fields: typing.ClassVar[tuple] = ("ph", "pc", "start", "end")
        @staticmethod
        def __new__(_cls, ph: str | int, pc: str | int, start, end):
            """
            Create new instance of IA_range(ph, pc, start, end)
            """

        @classmethod
        def _make(cls, iterable):
            """
            Make a new IA_range object from a sequence or iterable
            """

        def __getnewargs__(self):
            """
            Return self as a plain tuple.  Used by copy and pickle.
            """

        def __repr__(self):
            """
            Return a nicely formatted representation string
            """

        def _asdict(self):
            """
            Return a new dict which maps field names to their values.
            """

        def _replace(self, **kwds):
            """
            Return a new IA_range object replacing specified fields with new values
            """

    @classmethod
    def calculate_ph_txs_IA(
        cls, IA_ranges: List[IA_range], print_results: bool = False
    ) -> List[EquilibriumCalculationResult]:
        """
        Calculate the phase transitions given the specified incoming amount ranges.

        Parameters
        ----------
        IA_ranges
            list of `PhaseMapCalculation.IA_range` objects

        Returns
        -------
        List[EquilibriumCalculationResult]
            one equilibrium calculation result for the lower boundary, upper
            boundary and each phase transition
        """

    @classmethod
    def calculate_ph_txs_P(
        cls, P_min: float, P_max: float, print_results: bool = False
    ) -> List[EquilibriumCalculationResult]:
        """
        Calculate the phase transitions given the minimum and maximum pressure.

        Parameters
        ----------
        Pmin
            minimum pressure
        Pmax
            maximum pressure

        Returns
        -------
        List[EquilibriumCalculationResult]
            one equilibrium calculation result for the lower boundary, upper
            boundary and each phase transition

        """

    @classmethod
    def calculate_ph_txs_T(
        cls, T_min: float, T_max: float, print_results: bool = False
    ) -> List[EquilibriumCalculationResult]:
        """
        Calculate the phase transitions given the minimum and maximum temperature.

        Parameters
        ----------
        Tmin
            minimum temperature
        Tmax
            maximum temperature

        Returns
        -------
        List[EquilibriumCalculationResult]
            one equilibrium calculation result for the lower boundary, upper
            boundary and each phase transition

        """

class StreamCalculation(EquilibriumCalculationBase):
    streams: typing.ClassVar[dict] = {}
    result_type = chemapp.core.StreamCalculationResult

    @classmethod
    def get_eq_V(cls):
        """
        Get the system's volume at equilibrium.

        Note
        ----
        This function will always give the system volume. The
        previous behavior, the difference in volume, can be accessed
        using `get_eq_dV`.

        Returns
        -------
        float
            volume value

        """

    @classmethod
    def get_eq_dV(cls):
        """
        Get the difference in volume between input streams and equilibrium result (reaction volume).

        Returns
        -------
        float
            volume value

        """

    @classmethod
    def get_eq_G(cls):
        """
        Get the system's total gibbs energy at equilibrium.

        Note
        ----
        This function will always give the system total gibbs energy. The
        previous behavior, the difference in gibbs energy, can be accessed
        using `get_eq_dG`.

        Returns
        -------
        float
            total gibbs energy value

        """

    @classmethod
    def get_eq_dG(cls):
        """
        Get the difference in gibbs energy between input streams and equilibrium result (reaction gibbs energy).

        Returns
        -------
        float
            gibbs energy value

        """

    @classmethod
    def get_eq_S(cls):
        """
        Get the system's total entropy at equilibrium.

        Note
        ----
        This function will always give the system total entropy. The
        previous behavior, the difference in entropy, can be accessed
        using `get_eq_dS`.

        Returns
        -------
        float
            entropy value

        """

    @classmethod
    def get_eq_dS(cls):
        """
        Get the difference in entropy between input streams and equilibrium result (reaction entropy).

        Returns
        -------
        float
            entropy value

        """

    @classmethod
    def get_eq_H(cls):
        """
        Get the system's total enthalpy at equilibrium.

        Note
        ----
        This function will always give the system total enthalpy. The
        previous behavior, the difference in enthalpy, can be accessed
        using `get_eq_dH`.

        Returns
        -------
        float
            enthalpy value

        """

    @classmethod
    def get_eq_dH(cls):
        """
        Get the difference in enthalpy between input streams and equilibrium result (reaction enthalpy).

        Returns
        -------
        float
            enthalpy value

        """

    @classmethod
    def get_eq_Cp(cls):
        """
        Get the system's total heat capacity at equilibrium.

        Note
        ----
        This function will always give the system total heat capacity. The
        previous behavior, the difference in heat capacity, can be accessed
        using `get_eq_dCp`.

        Returns
        -------
        float
            heat capacity value

        """

    @classmethod
    def get_eq_dCp(cls):
        """
        Get the difference in heat capacity between input streams and equilibrium result (reaction heat capacity).

        Returns
        -------
        float
            heat capacity value

        """

    @classmethod
    def _set_eq_condition_ph(
        cls, variable: ConditionVariable, ph: str | int, value: float
    ) -> int:
        """
        Set an equilibrium condition on a phase.

        This method overrides the base method for use in stream calculations.

        Parameters
        ----------
        variable
            the variable to set
        ph
            phase name or zero-based index
        value
            the value to set the variable to

        Returns
        -------
        int
            equilibrium condition number


        """

    @classmethod
    def _set_eq_condition_sys(cls, variable: ConditionVariable, value: float) -> int:
        """
        Set a system-level equilibrium condition.

        This method overrides the base method for use in stream calculations.

        Parameters
        ----------
        variable
            the variable to set
                    value: the value to set the variable to

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def calculate_eq(
        cls,
        print_results: bool = False,
        estimate: bool = False,
        return_result: bool = False,
        **kwargs,
    ) -> None | StreamCalculationResult:
        """
        Do an equilibrium calculation.

        Any keyword arguments from `get_result_object()` can be added.

        Returns
        -------
        print_results:
            Indicates whether the results should be printed to
            console or not.
        estimate
            Indicates whether to use the previous calculation's
            results as an initial estimate.
        return_result
            Indicates whether the routine immediately returns a
            ResultObject.

        """

    @classmethod
    def calculate_eq_IA(
        cls, IA_ranges: List[Tuple[str, str, float, float]], print_results=False
    ):
        """
        Search for the target by adjusting the specified incoming amounts.

        Parameters
        ----------
        IA_ranges
            Tuples of (Phase name, Phase constituent name, minimum amount,
            maximum amount)
        print_results
            default: `False`. Indicates whether the results should be directly
            printed or not.

        """

    @classmethod
    def calculate_eq_P(cls, P_guess: float, print_results: bool = False) -> float:
        """
        Search for the target by adjusting pressure, and use the provided guess
        as a starting value.

        Parameters
        ----------
        P_guess
            pressure guess value to start search
        print_results
            default: `False`. Indicates whether the results should be directly
            printed or not.

        Returns
        -------
        float
            pressure

        """

    @classmethod
    def calculate_eq_T(cls, T_guess: float, print_results: bool = False) -> float:
        """
        Search for the target by adjusting temperature, and use the provided
        guess as a starting value.

        Parameters
        ----------
        T_guess
            temperature guess value to start search
        print_results
            default: `False`. Indicates whether the results should be directly
            printed or not.

        Returns
        -------
        float
            temperature

        """

    @classmethod
    def calculate_eq_V(cls, V_guess, print_results=False):
        """
        Search for the target by adjusting volume, and use the provided
        guess as a starting value.

        Parameters
        ----------
        V_guess
            volume guess value to start search
        print_results
            default: `False`. Indicates whether the results should be directly
            printed or not.

        Returns
        -------
        float
            volume

        """

    @classmethod
    def create_st(cls, name: str, T: float, P: float):
        """
        Create a stream and set its temperature and pressure.

        Parameters
        ----------
        name
            name of the new stream
        T
            temperature of the new stream
        P
            pressure of the new stream

        """

    @classmethod
    def get_result_object(
        cls, stream_extensive_properties: bool = False, error: bool = False
    ) -> StreamCalculationResult:
        """
        Extract an equilibrium calculation's results in the form of a
        `StreamCalculationResult` object.

        The created object contains all system wide results as well as for each
        phase and system component, statically. These are static and not
        affected by any changes to the ThermochemicalSystem or
        StreamCalculation classes.

        Refer to the `StreamCalculationResult` documentation for further
        information on how to use the results.

        Parameters
        ----------
        stream_extensive_properties
            include stream extensive properties in the result object

        Returns
        -------
        StreamCalculationResult
            An object that contains the result of the last stream calculation.

        """

    @classmethod
    def get_st_Cp(cls, name: str) -> float:
        """
        Get the specified stream's heat capacity.

        Parameters
        ----------
        name
            stream name

        Returns
        -------
        float
            heat capacity value

        """

    @classmethod
    def get_st_G(cls, name: str) -> float:
        """
        Get the specified stream's Gibbs energy.

        Parameters
        ----------
        name
            stream name

        Returns
        -------
        float
            Gibbs energy value

        """

    @classmethod
    def get_st_H(cls, name: str) -> float:
        """
        Get the specified stream's enthalpy.

        Parameters
        ----------
        name
            stream name

        Returns
        -------
        float
            enthalpy value

        """

    @classmethod
    def get_st_S(cls, name: str) -> float:
        """
        Get the specified stream's entropy.

        Parameters
        ----------
        name
            stream name

        Returns
        -------
        float
            entropy value

        """

    @classmethod
    def get_st_V(cls, name: str) -> float:
        """
        Get the specified stream's volume.

        Parameters
        ----------
        name
            stream name

        Returns
        -------
        float
            volume value

        """

    @classmethod
    def remove_st(cls, st: str):
        """
        Remove the specified stream from ChemApp.

        Parameters
        ----------
        st
            stream name

        """

    @classmethod
    def remove_sts(cls):
        """
        Remove all streams currently configured in ChemApp.

        """

    @classmethod
    def set_IA_pc(cls, name: str, ph: str | int, pc: str | int, value: float):
        """
        Set the amount of the specified phase constituent in the specified phase.

        Notice that this routine does not return the condition numbers.

        Parameters
        ----------
        name
            stream name
        ph
            phase name or index
        pc
            phase constituent name or index
        value
            amount value

        """

    @classmethod
    def set_st(cls, stream: StreamState, name: str | None = None, A: float = -1.0):
        """
        Set up a stream based on a stream object.

        A stream with the name ``name`` is created (or replaced), and the respective
        contents of the StreamState object are used as incoming amounts. The
        factor ``A`` is used to scale the incoming amounts. Pressure and
        temperature are set to the values of the ``stream`` object.

        It will set all incoming amounts that are present in the ``stream``
        object. However, incoming amounts already set for the stream (same name)
        are _not_ reset to zero.


        If name is given, the name of the stream object is not used.

        If A == -1.0 (or empty), the amount of the incoming stream object is used without
        modification.

        Parameters
        ----------
        stream
            stream object
        name
            stream name
        A
            the amount to add


        """

    @classmethod
    def set_target_formation_of_ph(cls, ph: str | int) -> int:
        """
        Set the target as formation of the specified phase.

        Parameters
        ----------
        ph
            phase name or index

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_target_precipitation_from_ph(cls, ph: str | int) -> int:
        """
        Set the target as precipitation from the specified phase.

        Parameters
        ----------
        ph
            phase name or index

        Returns
        -------
        int
            equilibrium condition number
        """

    @classmethod
    def set_tg_formation_of_ph(cls, ph: str | int) -> int:
        """
        Set the target as formation of the specified phase.
        FutureWarning: This function is renamed to `set_target_formation_of_ph`.
        `set_tg_formation_of_ph` will be removed in a future version.

        .. deprecated:: 826
            This function is renamed to `set_target_formation_of_ph`. `set_tg_formation_of_ph` will be removed in a future version.

        Parameters
        ----------
        ph
            phase name or index

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def set_tg_precipitation_from_ph(cls, ph: str | int) -> int:
        """
        Set the target as precipitation from the specified phase.

        FutureWarning: This function is renamed to `set_target_precipitation_from_ph`.
        `set_tg_precipitation_from_ph` will be removed in a future version.

        .. deprecated:: 826
            This function is renamed to `set_target_precipitation_from_ph`. `set_tg_precipitation_from_ph` will be removed in a future version.

        Parameters
        ----------
        ph : str | int
            phase name or index

        Returns
        -------
        int
            equilibrium condition number

        """

    @classmethod
    def stream_reaction_result(
        cls, streams: List[StreamState], T: float = 800.0, P: float = 1.0
    ) -> StreamCalculationResult:
        """
        Calculate the equilibrium for all streams at given temperatures and receive the result object directly.

        Parameters
        ----------
        streams : List[StreamState]
            streams to combine in calculation
        T : float = 800.0
            temperature to calculate at, default=800
        P : float = 1.0
            pressure to calculate at, default=1

        Returns
        -------
        StreamCalculationResult
            result object of the stream calculation


        """

class ThermochemicalSystem:
    """
    This class provides access to ChemApp functions that are used to perform
    tasks related to a loaded thermochemical system.

    A thermochemical system is described by a data file that is loaded into
    ChemApp. Two data file types are used, namely ASCII and transparent. ASCII
    data files have ``.dat`` extensions, and transparent data file have ``.cst``
    extensions. These files contain items such as system components, phases,
    phase constituents, thermochemical data, etc. Each transparent data file
    also contains a header, with more information about it, such as when it
    expires.

    The class is static, which means that you cannot create objects with it
    (e.g. ``ts = ThermochemicalSystem()``). The ``friendly`` module uses static
    classes as a convenient way to group ChemApp functions that perform related
    tasks together to make them easier to find and use.

    """

    @classmethod
    def _build_object(cls) -> ThermochemicalSystemConfig:
        """
        Extract the currently loaded thermochemical system's content into a `chemapp.core.ThermochemicalSystem` object.

        Returns
        -------
        ThermochemicalSystemConfig
            the configuration of the thermochemical system

        """

    @classmethod
    def get_config_pc(cls, ph: str | int, pc: str | int) -> PhaseConstituentConfig:
        """
        Get the full configuration details of the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        PhaseConstituentConfig
            configuration details object

        """

    @classmethod
    def get_config_pcs_in_ph(cls, ph: str | int) -> List[PhaseConstituentConfig]:
        """
        Get the full configuration details of all phase constituents in the
        specified phase as a list of PhaseConstituentConfig objects.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[PhaseConstituentConfig]
            configuration details objects

        """

    @classmethod
    def get_config_ph(cls, ph: str | int) -> PhaseConfig:
        """
        Get the full configuration details of the specified phase.

        The `PhaseConfig` object contains a list of all `PhaseConstituentConfig`
        objects.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        PhaseConfig
            configuration details object

        """

    @classmethod
    def get_config_phs(cls) -> List[PhaseConfig]:
        """
        Get the full configuration details of all phases as a list of PhaseConfig objects.

        Returns
        -------
        List[PhaseConfig]
            configuration details objects

        """

    @classmethod
    def get_config_sc(cls, sc: str | int) -> SystemComponentConfig:
        """
        Get the full configuration details of the specified system component.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        SystemComponentConfig
            configuration details object
        """

    @classmethod
    def get_config_scs(cls) -> List[SystemComponentConfig]:
        """
        Get the full configuration details of all system components as a list of SystemComponentConfig objects.

        Returns
        -------
        List[SystemComponentConfig]
            configuration details objects

        """

    @classmethod
    def get_count_pcs(cls) -> int:
        """
        Get the total number of phase constituents in the system.

        Returns
        -------
        int
            number of phase constituents

        """

    @classmethod
    def get_count_pcs_in_ph(cls, ph: str | int) -> int:
        """
        Get the number of constituents in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        int
            number of phase constituents in phase

        """

    @classmethod
    def get_count_phs(cls) -> int:
        """
        Get the number of phases.

        Returns
        -------
        int
            number of phases
        """

    @classmethod
    def get_count_scs(cls) -> int:
        """
        Get the number of system components.

        Returns
        -------
        int
            number of system components

        """

    @classmethod
    def get_cst_info_dict(cls) -> Dict[str, str | List[int] | int]:
        """
        Get information stored in the header of the currently loaded transparent file as a dictionary.

        The dictionary contains the items listed in the following table.

        +---------+------------+------------------------------------------------------------------------------------------+
        | Name    | Type       | Description                                                                              |
        +=========+============+==========================================================================================+
        | ver     | int        | version number of the transparent file header format                                     |
        +---------+------------+------------------------------------------------------------------------------------------+
        | nwp     | str        | name of the program which wrote the data-file (40 chars)                                 |
        +---------+------------+------------------------------------------------------------------------------------------+
        | vnw     | int[3]     | version number of the program which wrote the data-file                                  |
        +---------+------------+------------------------------------------------------------------------------------------+
        | nrp     | str        | ID(s) of the programs which are permitted to read the data-file (40 chars)               |
        +---------+------------+------------------------------------------------------------------------------------------+
        | vnr     | int[3]     | minimum version number of the reading program necessary to load the data-file            |
        +---------+------------+------------------------------------------------------------------------------------------+
        | dtc     | int[6]     | creation date of the data-file                                                           |
        +---------+------------+------------------------------------------------------------------------------------------+
        | dte     | int[6]     | expiry date of the data-file                                                             |
        +---------+------------+------------------------------------------------------------------------------------------+
        | id      | str        | user ID(s) of the license holder(s) who are permitted to read this data-file (255 chars) |
        +---------+------------+------------------------------------------------------------------------------------------+
        | usr     | str        | name(s) of the license holder(s) who are permitted to read this data-file (80 chars)     |
        +---------+------------+------------------------------------------------------------------------------------------+
        | rem     | str        | Returns the remark (80 chars)                                                            |
        +---------+------------+------------------------------------------------------------------------------------------+

        Returns
        -------
        Dict[str, str | List[int] | int]
            header information


        """

    @classmethod
    def get_cst_info_str(cls) -> str:
        """
        Print the information stored in the header of the currently loaded transparent file to a string.

        The data being used in this printable information str can be acquired
        via `get_cst_info_dict()`.

        Returns
        -------
        str
            Printable string representation.

        """

    @classmethod
    def get_index_pc(cls, ph: str | int, pc: str | int) -> int:
        """
        Get the index of the specified phase consituent in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        int
            number of phase constituents in phase

        """

    @classmethod
    def get_index_ph(cls, ph: str | int) -> int:
        """
        Get the index of the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        int
            index of phase
        """

    @classmethod
    def get_index_sc(cls, sc: str | int) -> int:
        """
        Get the index of the specified system component.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        int
            index of system component

        """

    @classmethod
    def get_mm_pc(cls, ph: str | int, pc: str | int) -> float:
        """
        Get the molar mass of the specified phase consituent in the specified
        phase.

        Notice that the result is in the units currently set. Typically, for a
        meaningful answer, changing the amount unit to grams is required. See class:`Units`

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        float
            molar mass

        """

    @classmethod
    def get_mm_pcs(cls) -> List[float]:
        """
        Get the molar masses of all phase constituents.

        Returns
        -------
        List[float]
            molar masses

        """

    @classmethod
    def get_mm_pcs_in_ph(cls, ph: str | int) -> List[float]:
        """
        Get the molar mass of the phase consituents in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[float]
            molar masses

        """

    @classmethod
    def get_mm_sc(cls, sc: str | int) -> float:
        """
        Get the molar mass of the specified system component.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        float
            molar mass

        """

    @classmethod
    def get_mm_scs(cls) -> List[float]:
        """
        Get the molar masses of all system components.

        Returns
        -------
        List[float]
            molar masses

        """

    @classmethod
    def get_model_ph(cls, ph: str | int) -> SolutionModel:
        """
        Get the model name of the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        SolutionModel
            enum of phase model

        """

    @classmethod
    def get_model_phs(cls) -> List[SolutionModel]:
        """
        Get the solution models of all phases.

        Returns
        -------
        List[SolutionModel]
            enums of phase models

        """

    @classmethod
    def get_name_pc(cls, ph: str | int, pc: str | int) -> str:
        """
        Get the name of the specified phase constituent in the specified phase.

        Yes you can use the name of the phase constituent for figuring out the
        name of the phase constituent.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        str
            phase constituent name

        """

    @classmethod
    def get_name_pcs(cls) -> List[str]:
        """
        Get the names of all phase constituents.

        Returns
        -------
        List[str]
            phase constituent names

        """

    @classmethod
    def get_name_pcs_in_ph(cls, ph: str | int) -> List[str]:
        """
        Get the names of all phase constituents in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[str]
            phase constituent names

        """

    @classmethod
    def get_name_ph(cls, ph: str | int) -> str:
        """
        Get the name of the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        str
            phase name

        """

    @classmethod
    def get_name_ph_pcs(cls) -> List[Tuple[str, str]]:
        """
        Get the names of all phases and phase constituents as a list of tuples.

        Returns
        -------
        List[Tuple[str,str]]
            phase name, phase constituents list

        """

    @classmethod
    def get_name_phs(cls) -> List[str]:
        """
        Get the names of all phases.

        Returns
        -------
        List[str]
            phase names

        """

    @classmethod
    def get_name_sc(cls, sc: str | int) -> str:
        """
        Get the name of the specified system component.

        Parameters
        ----------
        sc
            system component name or zero-based index

        Returns
        -------
        str
            system component name

        """

    @classmethod
    def get_name_scs(cls, include_electrons: bool = True) -> List[str]:
        """
        Get the names of all system components.

        This function has a parameter `include_electrons`. If set to `True`,
        the list of system components will include phase internal electrons.
        If set to `False`, the list of system components will exclude those.

        Parameters
        ----------
        include_electrons
            (bool, default=True) include electrons in the list of system components

        Returns
        -------
        List[str]
            system component names
        """

    @classmethod
    def get_object(cls) -> ThermochemicalSystemConfig:
        """
        Extract the currently loaded thermochemical system's content into a `ThermochemicalSystemConfig` object.

        Returns
        -------
        ThermochemicalSystemConfig
            configuration object

        """

    @classmethod
    def get_status_pc(cls, ph: str | int, pc: str | int) -> Status:
        """
        Get the status of the specified phase constituent in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index

        Returns
        -------
        Status
            status enumeration
        """

    @classmethod
    def get_status_pcs(cls) -> List[Status]:
        """
        Get the statuses of all phase constituents.

        Returns
        -------
        List[Status]
            status enumerations
        """

    @classmethod
    def get_status_pcs_in_ph(cls, ph: str | int) -> List[Status]:
        """
        Get the statuses of all phase constituents in the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        List[Status]
            status enumerations
        """

    @classmethod
    def get_status_ph(cls, ph: str | int) -> Status:
        """
        Get the status of the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index

        Returns
        -------
        Status
            status enumeration

        """

    @classmethod
    def get_status_phs(cls) -> List[Status]:
        """
        Get the statuses of all phases.

        Returns
        -------
        List[Status]
            status enumerations

        """

    @classmethod
    def get_stoich_pc(cls, ph: str | int, pc: str | int, scs=None) -> List[float]:
        """
        Get the stoichiometry of the specified phase consituent in the specified phase.

        The order of the factors for the system components is determined by the
        loaded thermodynamic data file.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index
        scs
            list of system components (names or zero-based indexes) to include
            in the result. (omission and None implies all)

        Returns
        -------
        List[float]
            stoichiometric factors

        """

    @classmethod
    def get_stoich_pcs(cls, scs: List[str] = None) -> List[List[float]]:
        """
        Get the stoichiometry of all phase constituents.

        The order of the factors for the system components as well as the order
        of the phase constituents is determined by the loaded thermodynamic data
        file.

        Parameters
        ----------
        scs
            list of system components (names or zero-based indexes) to include
            in the result. (omission and None implies all)

        Returns
        -------
        List[List[float]]
            phase constituents stoichiometries

        """

    @classmethod
    def get_stoich_pcs_in_ph(cls, ph: str | int, scs=None) -> List[List[float]]:
        """
        Get the stoichiometry of the phase consituents in the specified phase.

        The order of the factors for the system components as well as the order
        of the phase constituents is determined by the loaded thermodynamic data
        file.


        Parameters
        ----------
        ph
            phase name or zero-based index
        scs
            list of system components (names or zero-based indexes) to include
            in the result. (omission and None implies all)

        Returns
        -------
        List[List[float]]
            phase constituent stoichiometries
        """

    @classmethod
    def get_stoich_sc(cls, sc: str | int, scs: List[str] | None = None) -> List[float]:
        """
        Get the stoichiometry of the specified system component.

        Since switching system components is discouraged, this routine is rarely
        applicable.

        Parameters
        ----------
        sc
            system component name or zero-based index
        scs
            list of system components (names or zero-based indexes) to include
            in the result. Omission and None implies all.

        Returns
        -------
        List[float]
            system component stoichiometry

        """

    @classmethod
    def get_stoich_scs(cls, scs=None) -> List[List[float]]:
        """
        Get the stoichiometry of all system components.

        Parameters
        ----------
        scs
            list of system components (names or zero-based indexes) to include
            in the result. Omission and None implies all.

        Returns
        -------
        List[List[float]]
            system components stoichiometries

        """

    @classmethod
    def get_str_phs(cls) -> str:
        """
        Print a list of the phases in the loaded thermochemical system to a
        string.

        Returns
        -------
        str
            phases, printable

        """

    @classmethod
    def get_str_phs_pcs(cls) -> str:
        """
        Print a list of phases and phase constituents in the loaded
        thermochemical system to a string."

        Returns
        -------
        str
            phases and phase constituents, printable

        """

    @classmethod
    def get_str_scs(cls) -> str:
        """
        Print a list of the system components in the loaded thermochemical
        system to a string.

        Returns
        -------
        str
            system components, printable

        """

    @classmethod
    def get_y_sc_pc(cls, ph: str | int, pc: str | int, sc: str | int) -> float:
        """
        Get the mass fraction of the specified system component in the specified
        phase constituent.

        Parameters
        ----------

        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index
        sc
            system component name or zero-based index

        Returns
        -------
        float
            mass fraction

        """

    @classmethod
    def get_y_sc_pcs(cls, sc: str | int) -> List[float]:
        """
        Get the mass fraction of the specified system component in all phase
        constituents.

        Parameters
        ----------
        sc
            system component name or zero-based index (str or float)

        Returns
        -------
        List[float]
            mass fractions

        """

    @classmethod
    def get_y_sc_pcs_in_ph(cls, ph: str | int, sc: str | int) -> List[float]:
        """
        Get the mass fraction of the specified system component in the
        constituent of the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index
        sc
            system component name or zero-based index

        Returns
        -------
        List[float]
            mass fractions

        """

    @classmethod
    def get_y_scs_pc(
        cls, ph: str | int, pc: str | int, scs: List[str | float] | None = None
    ) -> List[float]:
        """
        Get the mass fractions of the specified system components in the
        specified phase constituent.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index
        scs
            list of system components (names or zero-based indexes) to
            include in the result. (omission and None implies all)

        Returns
        -------
        List[float]
            mass fractions

        """

    @classmethod
    def get_y_scs_pcs(cls, scs: List[str | float] | None = None) -> List[List[float]]:
        """
        Get the mass fractions of the specified system components in all phase
        constituents.

        Parameters
        ----------
        scs
            list of system components (names or zero-based indexes) to
            include in the result. (omission and None implies all)


        Returns
        -------
        List[List[float]]
            mass fractions

        """

    @classmethod
    def get_y_scs_pcs_in_ph(
        cls, ph: str | int, scs: List[str | float] | None = None
    ) -> List[List[float]]:
        """
        Get the mass fractions of the specified system components in the
        constituents of the specified phase.

        Parameters
        ----------
        ph
            phase name or zero-based index
        scs
            list of system components (names or zero-based indexes) to include
            in the result. (omission and None implies all)

        Returns
        -------
        List[List[float]]
            mass fractions

        """

    @classmethod
    def load(cls, file_path: str | PathLike):
        """
        Initialise ChemApp and load the thermochemical data in the specified
        file.

        The function performs these steps:

        * Initialise ChemApp (:py:meth:`~chemapp.basic.tqini`)
        * Prevent errrors from being written to the console (:py:meth:`~chemapp.basic.tqcio`)
        * Open, read and close the file

        Take note of the default units after initialisation:

        Pressure:    bar
        Volume:      dm3
        Temperature: K
        Energy:      J
        Amount:      mol

        Parameters
        ----------
        file_path
            file path, can be a string or a Path object

        """

    @classmethod
    def set_status_pc(cls, ph: str | int, pc: str | int, status: Status):
        """
        Set the status of the specified phase constituent in the specified
        phase to the specified value.

        Parameters
        ----------
        ph
            phase name or zero-based index
        pc
            phase constituent name or zero-based index
        status
            new status (ENTERED, DORMANT or ELIMINATE. `Status` enumeration)

        """

    @classmethod
    def set_status_pcs(cls, status: Status):
        """
        Set the statuses of all phase constituents to the specified value.

        Parameters
        ----------
        status
            new status (ENTERED, DORMANT or ELIMINATE. `Status` enumeration)

        """

    @classmethod
    def set_status_pcs_in_ph(cls, ph: str | int, status: Status):
        """
        Set the statuses of all phase constituents in the specified phase to
        the specified value.

        Parameters
        ----------
        ph
            phase name or zero-based index
        status
            new status (ENTERED, DORMANT or ELIMINATE. `Status` enumeration)

        """

    @classmethod
    def set_status_ph(cls, ph: str | int, status: Status):
        """
        Set the status of the specified phase to the specified value.

        Parameters
        ----------
        ph
            phase name or zero-based index
        status
            new status (ENTERED, DORMANT or ELIMINATE. `Status` enumeration)

        """

    @classmethod
    def set_status_phs(cls, status: Status):
        """
        Set the status of all phases to the specified value.

        This routine can be used to reset the status of all phases.

        Parameters
        ----------
        status
            new status (ENTERED, DORMANT or ELIMINATE. `Status` enumeration)

        """

class Units:
    """

    This class provides access to ChemApp functions that are used to perform
    tasks related unit of measure.

    The class is static, which means that you cannot create objects with it
    (e.g. ``units = Units()``). The ``friendly`` module uses static classes as a
    convenient way to group ChemApp functions that perform related tasks
    together to make them easier to find and use.

    """

    @classmethod
    def get(
        cls,
    ) -> Dict[
        str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit
    ]:
        """
        Get ChemApp's currently selected units of measure.

        The keys are 'P', 'V', 'T', 'E', 'A' for pressure, volume, temperature,
        energy and amount.

        The values are PressureUnit, VolumeUnit, TemperatureUnit, EnergyUnit or
        AmountUnit enums.

        Returns
        -------
        Dict[str, PressureUnit | VolumeUnit | TemperatureUnit | EnergyUnit | AmountUnit]
            currently selected units

        """

    @classmethod
    def get_A_unit(cls) -> AmountUnit:
        """
        Get the amount unit of measure.

        Returns
        -------
        AmountUnit
            current amount unit

        """

    @classmethod
    def get_E_unit(cls) -> EnergyUnit:
        """
        Get the energy unit of measure.

        Returns
        -------
        EnerygUnit
            current energy unit

        """

    @classmethod
    def get_P_unit(cls) -> PressureUnit:
        """
        Get the pressure unit of measure.

        Returns
        -------
        PressureUnit
            current pressure unit

        """

    @classmethod
    def get_T_unit(cls) -> TemperatureUnit:
        """
        Get the temperature unit of measure.

        Returns
        -------
        TemperatureUnit
            current temperature unit

        """

    @classmethod
    def get_V_unit(cls) -> VolumeUnit:
        """
        Get the volume unit of measure.

        Returns
        -------
        VolumeUnit
            current volume unit
        """

    @classmethod
    def get_str(cls) -> str:
        """
        Print the current unit of measure used by ChemApp to a string.

        Returns
        -------
        str
            printable Units.
        """

    @classmethod
    def set(
        cls,
        P: PressureUnit | None = ...,
        V: VolumeUnit | None = ...,
        T: TemperatureUnit | None = ...,
        E: EnergyUnit | None = ...,
        A: AmountUnit | None = ...,
    ):
        """
        Set ChemApp's unit of measure.

        Using enumerations: PressureUnit, VolumeUnit, TemperatureUnit,
        EnergyUnit, AmountUnit

        Parameters
        ----------
        P
            units for pressure (default 'atm')
        V
            units for volume (default 'm3')
        T
            units for temperature (default 'C')
        E
            units for energy (default 'kWh')
        A
            units for amount (default 'kg')

        """

    @classmethod
    def set_A_unit(cls, unit: AmountUnit):
        """
        Set the amount unit of measure.

        Parameters
        ----------
        AmountUnit
            new amount unit

        """

    @classmethod
    def set_E_unit(cls, unit: EnergyUnit):
        """
        Set the energy unit of measure.

        Parameters
        ----------
        EnergyUnit
            new energy unit


        """

    @classmethod
    def set_P_unit(cls, unit: PressureUnit):
        """
        Set the pressure unit of measure.

        Parameters
        ----------
        PressureUnit
            new pressure unit

        """

    @classmethod
    def set_T_unit(cls, unit: TemperatureUnit):
        """
        Set the temperature unit of measure.

        Parameters
        ----------
        TemperatureUnit
            new temperature unit

        """

    @classmethod
    def set_V_unit(cls, unit: VolumeUnit):
        """
        Set the volume unit of measure.

        Parameters
        ----------
        VolumeUnit
            new volume unit

        """

    @classmethod
    def use_chemapp_defaults(cls):
        """
        Reset ChemApp to use the default units.

        P: atm, T: C, V: m3, E: kWh, A: kg

        """
